/*************************************************************************
  *
  * ADOBE CONFIDENTIAL
  * __________________
  *
  *  Copyright 2012 Adobe Systems Incorporated
  *  All Rights Reserved.
  *
  * NOTICE:  All information contained herein is, and remains
  * the property of Adobe Systems Incorporated and its suppliers,
  * if any.  The intellectual and technical concepts contained
  * herein are proprietary to Adobe Systems Incorporated and its
  * suppliers and may be covered by U.S. and Foreign Patents,
  * patents in process, and are protected by trade secret or copyright law.
  * Dissemination of this information or reproduction of this material
  * is strictly forbidden unless prior written permission is obtained
  * from Adobe Systems Incorporated.
  **************************************************************************/
package com.adobe.cq.dms.tagmanager;

import java.util.Set;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.commons.json.JSONException;
import org.apache.sling.commons.json.JSONObject;

/**
 * The implementors of a <code>TagHandler</code> are responsible for handling a
 * specific type of tag.
 */
public interface TagHandler {
    
    /**
     * Returns the {@TagType} of this <code>TagHandler</code>.
     * 
     * @param Tag resource (normally a configuration page).
     * @return Tag type
     */
    TagType getType(Resource resource) {
		return null;
	}
    
    /**
     * Returns the template path this TagHandler is registered for.
     * 
     * @return
     */
    String getTemplatePath() {
		return null;
	}
    
    /**
     * Returns one (or more) tag JSONObjects. Depending on the <tt>type</tt>
     * either <tt>client_js</tt> or <tt>script_url</tt> needs to be set but not
     * both. The property <tt>product_config</tt> is only used in conjunction
     * with <tt>product_??</tt> types and contains information used to
     * re-generate the core/client_js. 
     * <p>The format of the different product configuration is as follows.
     * <ul>
     * <li>SiteCatalyst
     * <pre>
     * {
     *     "sc_account": ["bugzilla_dal"],
     *     "sc_instance_variable": "s",
     *     "sc_tracking_server": "adobetagmanagerengineering.d2.sc.omtrdc.net",
     *     "sc_charset": "UTF-8"
     * }
     * </pre>
     * </li>
     * <li>SiteCatalyst Media
     * <pre>
     * {
     *     "sc_account": ["bugzilla_dal"],
     *     "sc_instance_variable": "s"
     * }
     * </pre>
     * </li>
     * <li>SiteCatalyst Survey
     * <pre>
     * {
     *     "sc_instance_variable": "s"
     * }
     * </pre>
     * </li>
     * <li>SiteCatalyst Partner
     * <pre>
     * {
     *     "sc_instance_variable": "s"
     * }
     * </pre>
     * </li>
     * <li>Adobe Target
     * <pre>
     * {
     *     "tt_client_code": "tagmanagereng"
     * }
     * </pre>
     * </li>
     * 
     * @param resource Tag resource (normally a configuration page).
     * @return A {@link Set} of tag JSONObjects in the following JSON format:
     * <pre>
     * {
     *      "type" : "custom",
     *      "description" : "some description",
     *      "client_js" : "var i=0;",
     *      "script_url" : "http://localhost/scripts/script.js",
     *      "product_config": {
     *          "tt_client_code" : "myclientcode"
     *      },
     *      "trigger_conditions": {
     *          "conditional_start_timestamp" : 123456,
     *          "conditional_end_timestamp" : 123456,
     *          "conditional_match_hostname" : "localhost",
     *          "conditional_match_path" : "/my/path.html",
     *          "conditional_match_querystring_key" : "queryparameter",
     *          "conditional_match_querystring_value" : "queryparameter value",
     *          "conditional_match_cookie_key" : "cookiename",
     *          "conditional_match_cookie_value" : "cookie value"
     *      }
     * }
     * </pre>
     * @throws JSONException
     */
    Set<JSONObject> getTags(Resource resource) throws JSONException {
		return null;
	}

}
