/*************************************************************************
  *
  * ADOBE CONFIDENTIAL
  * __________________
  *
  *  Copyright 2012 Adobe Systems Incorporated
  *  All Rights Reserved.
  *
  * NOTICE:  All information contained herein is, and remains
  * the property of Adobe Systems Incorporated and its suppliers,
  * if any.  The intellectual and technical concepts contained
  * herein are proprietary to Adobe Systems Incorporated and its
  * suppliers and may be covered by U.S. and Foreign Patents,
  * patents in process, and are protected by trade secret or copyright law.
  * Dissemination of this information or reproduction of this material
  * is strictly forbidden unless prior written permission is obtained
  * from Adobe Systems Incorporated.
  **************************************************************************/
package com.adobe.cq.dms.tagmanager;

import org.apache.sling.commons.json.JSONArray;
import org.apache.sling.commons.json.JSONObject;

import com.day.cq.wcm.webservicesupport.Configuration;

/**
 * The <code>TagManagerService</code> wraps {@link SitecatalystHttpClient} and
 * implements the TagManager related API methods.
 */
public interface TagManagerService {
    
    /**
     * Initializes a bucket which is an entity to hold revisions of a container.
     * A bucket is immutable and has the 3 environments; development, stage,
     * live.
     * 
     * @param configuration Service configuration
     * @param name Bucket name consisting only of alphanumeric or underscore
     *            characters.
     * @param description A description
     * @return The initialized bucket JSONObject with the following JSON format:
     * <pre>
     * {
     *      "bucket_id": 123,
     *      "bucket_description": "A bucket",
     *      "bucket_name": "bucket1"
     * }
     * </pre>
     * @throws TagManagerException if bucket can not be created (i.e. max.
     *             numbers of buckets is reached)
     */
    JSONObject initializeBucket(Configuration configuration, String name, String description) throws TagManagerException {
		return null;
	}

    /**
     * Marks a bucket as deleted.
     * 
     * @param configuration Service configuration
     * @param bucketId Bucket identifier
     * @return The deleted bucket JSONObject with the following JSON format:
     * <pre>
     * {
     *      "bucket_id": 123,
     *      "bucket_description": "A bucket",
     *      "bucket_name": "bucket1"
     * }
     * </pre>
     * @throws TagManagerException if an error occurs during the operation
     */
    JSONObject deleteBucket(Configuration configuration, long bucketId) throws TagManagerException {
		return null;
	}
    
    /**
     * Returns a list of initialized buckets.
     * 
     * @param configuration Service configuration
     * @return A JSONArray of bucket JSONObjects, see
     *         {@link com.adobe.cq.dms.tagmanager.TagManager#initializeBucket(String, String)}
     *         for JSON format.
     * @throws TagManagerException if an error occurs during the operation
     */
    JSONArray getBuckets(Configuration configuration) throws TagManagerException {
		return null;
	}
    
    /**
     * Returns a container JSONObject of the provided <code>revisionId</code>.
     * If no <code>revisionId</code> is provided the latest revision will be
     * returned.
     * 
     * @param configuration Service configuration
     * @param bucketId ID of the bucket
     * @param environment Environment
     * @param revisionId Revision ID or <code>null</code> to get the latest
     *            revision
     * @return A container JSOBObject of the following format.
     * <pre>
     * {
     *      "bucket_id": 123, 
     *      "environment": "live", 
     *      "revision_id": 4, 
     *      "status": "deployed", 
     *      "created_at": "datetime", 
     *      "deployed_at": "datetime", 
     *      "deployed_url": "http://...", 
     *      "js_before": "userid", 
     *      "change_login": 122, 
     *      "change_note": "Change x from dev", 
     *      "tags": [
     *          {
     *              "description": "a desc", 
     *              "script_url": "location of script", 
     *              "client_js": "The javascript source of the tag", 
     *              "trigger_conditions": {
     *                  "bar": "b", 
     *                  "key": "a"
     *              }, 
     *              "type": "product_sc"
     *          } 
     *      ]
     *  } 
     *  </pre>
     *  @throws TagManagerException if an error occurs during the operation
     */
    JSONObject getContainer(Configuration configuration, Long bucketId, Environment environment, Long revisionId) throws TagManagerException {
		return null;
	}
    
    /**
     * Saves a new revision of a container in the specified environment. The container
     * will be completely replaced with the contents of this new revision.
     * 
     * @param configuration Service configuration
     * @param container Container JSONObject to save in the format:
     * <pre>
     * {
     *      "bucket_id": 123, 
     *      "environment": "live", 
     *      "deploy": true, 
     *      "js_before": "userid", 
     *      "change_note": "Change x from dev", 
     *      "tags": [
     *          {
     *              "description": "a desc", 
     *              "script_url": "location of script", 
     *              "client_js": "The javascript source of the tag", 
     *              "trigger_conditions": { "...": "..." },
     *              "type": "product_sc"
     *          }
     *      ]
     * }
     * </pre>
     * @return A container JSONObject of the following format.
     * <pre>
     * {
     *      "bucket_id": 123, 
     *      "environment": "live", 
     *      "revision_id": 4, 
     *      "status": "deployed", 
     *      "created_at": "datetime", 
     *      "deployed_at": "datetime", 
     *      "deployed_url": "http://...", 
     *      "js_before": "userid", 
     *      "change_login": 122, 
     *      "change_note": "Change x from dev", 
     *      "tags": [
     *          {
     *              "description": "a desc", 
     *              "script_url": "location of script", 
     *              "client_js": "The javascript source of the tag", 
     *              "trigger_conditions": {
     *                  "bar": "b", 
     *                  "key": "a"
     *              }, 
     *              "type": "product_sc"
     *          } 
     *      ]
     *  } 
     *  </pre>
     *  @throws TagManagerException if an error occurs during the operation
     */
    JSONObject saveContainer(Configuration configuration, JSONObject container) throws TagManagerException {
		return null;
	}
    
    /**
     * Returns all revisions of a container for the specified environment. The 
     * method returns only a lightweight set of container JSONObjects.
     * 
     * @param configuration Service configuration
     * @param historydescription A history description in the following format:
     * <pre>
     * {
     *      "bucket_id": 123, 
     *      "environment": "live",
     *      "limit": 10,
     *      "offset": 4,
     *      "only_deployed": true
     * }
     * </pre>
     * @return A JSONArray of lightweight container JSONObjects in the format:
     * <pre>
     * {
     *      "bucket_id": 123, 
     *      "environment": "live", 
     *      "revision_id": 4, 
     *      "status": "deployed", 
     *      "created_at": "datetime", 
     *      "deployed_at": "datetime", 
     *      "deployed_url": "http://...", 
     *      "change_login": 122, 
     *      "change_note": "Change x from dev", 
     * } 
     * </pre>
     * @throws TagManagerException if an error occurs during the operation
     */
    JSONArray getContainerHistory(Configuration configuration, JSONObject historydescription) throws TagManagerException {
		return null;
	}
    
    /**
     * Copies a container into another (or the same) environment and allows to change
     * the deployment status of a container.
     * 
     * @param configuration Service configuration
     * @param containerdesc copy description in the format:
     * <pre>
     * {
     *      "bucket_id": 123, 
     *      "source_environment": "stage", 
     *      "revision_id": 10
     *      "destination_environment": "live", 
     *      "deploy": 1, 
     * }
     * </pre>
     * @return A container JSOBObject of the following format.
     * <pre>
     * {
     *      "bucket_id": 123, 
     *      "environment": "live", 
     *      "revision_id": 4, 
     *      "status": "deployed", 
     *      "created_at": "datetime", 
     *      "deployed_at": "datetime", 
     *      "deployed_url": "http://...", 
     *      "js_before": "userid", 
     *      "change_login": 122, 
     *      "change_note": "Change x from dev", 
     *      "tags": [
     *          {
     *              "type": "product_sc",
     *              "description": "a desc", 
     *              "script_url": "location of script", 
     *              "client_js": "The javascript source of the tag", 
     *              "trigger_conditions": {
     *                  "bar": "b", 
     *                  "key": "a"
     *              } 
     *          } 
     *      ]
     *  } 
     *  </pre>
     *  @throws TagManagerException if an error occurs during the operation
     */
    JSONObject copyContainer(Configuration configuration, JSONObject copydescription) throws TagManagerException {
		return null;
	}
    
    /**
     * Returns the container file contents.
     * 
     * @param configuration Service configuration
     * @param bucketId Bucket identifier
     * @param environment TagManager environment
     * @param revisionId Revision identifier
     * @return A string representation of the container file
     * @throws TagManagerException if an error occurs during the operation
     */
    String getContainerFile(Configuration configuration, Long bucketId, Environment environment, Long revisionId) throws TagManagerException {
		return null;
	}
    
    /**
     * Returns the container loader file contents.
     * 
     * @param configuration Service configuration
     * @param bucketId Bucket identifier
     * @return A string representation of the container loader file
     * @throws TagManagerException if an error occurs during the operation
     */
    String getContainerLoaderFile(Configuration configuration, Long bucketId) throws TagManagerException {
		return null;
	}
    
    /**
     * Returns the SiteCatalyst tag object.
     * 
     * @param configuration Service configuration
     * @param reportsuite SiteCataylst report suite ID
     * @param variablename SiteCatalyst variable name or <code>null</code>
     * @param trackingserver Tracking server URL or <code>null</code>
     * @param charset SiteCatalyst charset or <code>null</code>
     * @param currencyCode SiteCatalyst currency code or <code>null</code>
     * @return Tag JSONObject in the format:
     * <pre>
     * {
     *      "type": "product_sc",
     *      "client_js": "some javascript in here ...",
     *      "product_config": {
     *          "regenerate_core": true,
     *          "sc_version": "25.1",
     *          "sc_account": "bugzilla_dal",
     *          "sc_instance_variable": "s",
     *          "sc_tracking_server": "adobetagmanagerengineering.d2.sc.omtrdc.net",
     *          "sc_tracking_server_secure": "adobetagmanagerengineering.d2.sc.omtrdc.net",
     *          "sc_charset": "UTF-8",
     *          "sc_currency_code": ""
     *      }
     * }
     * </pre>
     * @throws TagManagerException if an error occurs during the operation
     */
    JSONObject getSCConfig(Configuration configuration, String reportsuite, String variablename, String trackingserver, String charset, String currencyCode) throws TagManagerException {
		return null;
	}
    
    /**
     * Returns the Media Module tag object.
     * 
     * @param configuration Service configuration
     * @param reportSuiteID SiteCatalyst ReportSuite ID
     * @param variableName SiteCatalyst instance variable name or
     *            <code>null</code>
     * @return Tag JSONObject in the format:
     * <pre>
     * {
     *      "type": "product_sc_media",
     *      "client_js": "some javascript in here ...",
     *      "product_config": {
     *          "regenerate_core": true
     *      }
     * }
     * </pre>
     * @throws TagManagerException if an error occurs during the operation
     */
    JSONObject getSCMediaConfig(Configuration configuration, String reportSuiteID, String variableName) throws TagManagerException {
		return null;
	}
    
    /**
     * Return the Survey Module tag object.
     * 
     * @param configuration Service configuration
     * @param variableName SiteCatalyst instance variable name or <code>null</code>
     * @return Tag JSONObject in the format:
     * <pre>
     * {
     *      "type": "product_sc_survey",
     *      "client_js": "some javascript in here ...",
     *      "product_config": {
     *          "regenerate_core": true
     *      }
     * }
     * </pre>
     * @throws TagManagerException if an error occurs during the operation
     */
    JSONObject getSCSurveyConfig(Configuration configuration, String variableName) throws TagManagerException {
		return null;
	}
    
    /**
     * Returns the Dependent JavaScript Code tag object.
     * 
     * @param configuration Service configuration
     * @param variableName SiteCatalyst instance variable name or <code>null</code>
     * @return Tag JSONObject in the format:
     * <pre>
     * {
     *      "type": "product_sc_partner",
     *      "client_js": "some javascript in here ...",
     *      "product_config": {
     *          "regenerate_core": true
     *      }
     * }
     * </pre>
     * @throws TagManagerException if an error occurs during the operation
     */
    JSONObject getSCPartnerConfig(Configuration configuration, String variableName) throws TagManagerException {
		return null;
	}
    
    /**
     * Returns the Adobe Target tag object.
     * 
     * @param configuration Service configuration
     * @param clientCode Adobe Target client code
     * @return Tag JSONObject in the format: 
     * <pre>
     * {
     *      "type": "product_tt",
     *      "client_js": "some javascript in here ...",
     *      "product_config": {
     *          "regenerate_core": true,
     *          "tt_client_code": "tagmanagereng"
     *      }
     * }
     * </pre>
     * @throws TagManagerException if an error occurs during the operation
     */
    JSONObject getTTConfig(Configuration configuration, String clientCode) throws TagManagerException {
		return null;
	}
    
    /**
     * Returns the {@link Environment} configured for this service.
     * 
     * @return
     */
    Environment getEnvironment() {
		return null;
	}
    
}
