/*
 * ***********************************************************************
 *
 *  ADOBE CONFIDENTIAL
 *  ___________________
 *
 *   Copyright 2012 Adobe Systems Incorporated
 *   All Rights Reserved.
 *
 *  NOTICE:  All information contained herein is, and remains
 *  the property of Adobe Systems Incorporated and its suppliers,
 *  if any.  The intellectual and technical concepts contained
 *  herein are proprietary to Adobe Systems Incorporated and its
 *  suppliers and are protected by trade secret or copyright law.
 *  Dissemination of this information or reproduction of this material
 *  is strictly forbidden unless prior written permission is obtained
 *  from Adobe Systems Incorporated.
 * ************************************************************************
 */

package com.adobe.cq.mcm.silverpop;


import java.util.Date;
import java.util.List;
import java.util.Map;

import com.day.cq.mcm.emailprovider.types.Account;
import com.day.cq.mcm.emailprovider.types.Email;
import com.day.cq.mcm.emailprovider.types.EmailServiceActions;
import com.day.cq.mcm.emailprovider.types.ListAttribute;
import com.day.cq.mcm.emailprovider.types.PersonalizationInfo;
import com.day.cq.mcm.emailprovider.types.SendClassification;
import com.day.cq.mcm.emailprovider.types.Subscriber;
import com.day.cq.mcm.emailprovider.types.SubscriptionList;
import com.day.cq.wcm.foundation.forms.FieldDescription;
import com.day.cq.wcm.webservicesupport.Configuration;
import com.adobe.cq.mcm.silverpop.impl.DirectoryContentIDS;


/**
 * SilverPop Webservice client that makes webservice calls to SilverPop WebService endpoint.
 */
public interface SilverPopClient {
    /**
     * This function calls a webservice that describes the silverpop subscriber parameters.
     * @param listId The list associated with the personalization
     * @return 
     * @throws SilverPopException In case of error.
     */
    public PersonalizationInfo describeSubscriber(String listId) throws SilverPopException {
		return null;
	}
    /**
     * @deprecated since 5.6.1
     * @see {@link SilverPopClient#getListAttributes(String) getListAttributes}
     * This function fetches the list of form fields needed while executing the given formAction on associated list
     * @param listId The list associated with the form
     * @param formAction {Subscribe, Unsubscribe, AutoResponder}
     * @return A list of FieldDescription needed to execute the given formAction on associated list.
     * @throws SilverPopException
     */
    @Deprecated 
    public List<FieldDescription> getFormFields(String listId, EmailServiceActions formAction) throws SilverPopException {
		return null;
	}
    /**
     * 
     * @param listId The list or database whose attributes are to be fetched
     * @return The list of {@link ListAttribute attributes} in the list 
     * @throws SilverpopException
     */
    public List<ListAttribute> getListAttributes(String listId) throws SilverPopException {
		return null;
	}
    /**
     * This function calls an silverpop webservice to fetch the subscription lists in silverpop.
     * @param filterMap A Map containing custom filtering parameters to fetch the list.
     * @return The list of mailing lists available in silverpop.
     * @throws SilverPopException In case of error.
     */
    public List<SubscriptionList> getSubscriptionList(Map<String, Object> filterMap)
            throws SilverPopException {
		return null;
	}

    /**
     * This function calls an silverpop webservice to add a subscriber to contact lists.
     * @param subscriber A subscriber that needs to be added to the mailing lists.
     * @param lists An array of subscription lists to which this subscriber needs to be added. 
     * The lists should share the same parent database at SilverPop. If lists are null, the subscriber will be added to the parentDB only.
     * @param parentDB Id of parent database of the lists
     * @throws SilverPopException In case of error.
     */
    public void addSubscriberToList(Subscriber subscriber, SubscriptionList[] lists, String parentDB)
            throws SilverPopException {
	}

    /**
     * This fucntion calls an silverpop webservice to publish and Email to silverpop.
     * @param email The email object to be published
     * @return the published email with new properties saved on external email provider
     * @throws SilverPopException In case of error.
     */
    public Email publishEmail(Email email) throws SilverPopException {
		return null;
	}
    
    /**
     * This function calls an silverpop webservice to update published email on silverpop
     * @param email The email to be updated
     * @throws SilverPopException
     */
    public void updateEmail(Email email) throws SilverPopException {
	}

    /**
     * This function calls an silverpop webservice to send an email through silverpop.
     * @param email The email to be sent
     * @param lists The lists from which the subscriber will be removed. These lists must share the same
     * database at SilverPop.
     * @param parentDB The parent database associated with the lists. If lists are null the subscriber will be
     * deleted from the database and all associated contact lists.
     * @throws SilverPopException In case of error.
     */

    public void deleteSubscriberFromList(Subscriber subscriber, SubscriptionList[] lists, String parentDB)
            throws SilverPopException {
	}

    
    public List<Account> getAccounts(Map<String, Object> filterMap)
            throws SilverPopException {
		return null;
	}
    /**
     * This function returns the Directory Content IDs for an Engage Org at SilverPop corresponding to type
     * @param type The type of Directory Content whose id is to be retrieved
     * @return Id of the specified Directory Content
     * @throws SilverPopException in case of errors
     */
	public String getDirectoryConetentId(DirectoryContentIDS type) throws SilverPopException {
		return null;
	}
    /**
     * This function is used to validate the connection parameters with SilverPop provided while creating the ExactTarget cloudservice configuration
     * @throws SilverPopException in case of errors.
     */
	public void checkCredentials() throws SilverPopException {
	}
	
    public List<MailingTemplate> getMailingTemplates(GetMailingTemplateParams params) throws SilverPopException {
		return null;
	}
    
    public void scheduleMailing(ScheduleMailingParams params) throws SilverPopException {
	}	
}
