/*******************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 ******************************************************************************/
package com.adobe.cq.sightly;

import javax.script.Bindings;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.SlingHttpServletResponse;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ValueMap;
import org.apache.sling.api.scripting.SlingBindings;
import org.apache.sling.api.scripting.SlingScriptHelper;
import org.apache.sling.scripting.sightly.pojo.Use;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import aQute.bnd.annotation.ConsumerType;
import com.day.cq.wcm.api.Page;
import com.day.cq.wcm.api.PageManager;
import com.day.cq.wcm.api.components.Component;
import com.day.cq.wcm.api.designer.Design;
import com.day.cq.wcm.api.designer.Designer;
import com.day.cq.wcm.api.designer.Style;

/**
 * Abstract implementation of {@link Use} interface. This could be extended to provide custom Use models.
 **/
@ConsumerType
public abstract class WCMUsePojo implements Use {

    private static final Logger LOG = LoggerFactory.getLogger(WCMUsePojo.class);

    private Bindings bindings;

    /**
     * Initialize bindings and calls #activate()
     *
     * @param scriptBindings Bindings to be used, there is no guarantee of having any particular bindings available.
     *
     * */
    public final void init(Bindings scriptBindings) {
	}

    /**
     * Implement this method to perform post initialization tasks.
     * This is called from the WCMUse#init
     *
     * @throws Exception in case of any error during activation
     *
     * */
    public abstract void activate() throws Exception {
	}

    /**
     * Get an object associated with the given name
     *
     * @param name  Object property name
     * @param type  Expected object type
     *
     * @return Object or null if Object cannot be found or typed.
     *
     * */
    public final <T> T get(String name, Class<T> type) {
		return null;
	}

    /**
     * @return {@link WCMBindings#WCM_MODE} binding if available, otherwise null is returned
     * */
    public final SightlyWCMMode getWcmMode() {
		return null;
	}

    /**
     * @return {@link WCMBindings#PAGE_MANAGER} binding if available, otherwise null is returned
     * */
    public final PageManager getPageManager() {
		return null;
	}

    /**
     * @return {@link WCMBindings#CURRENT_PAGE} binding if available, otherwise null is returned
     * */
    public final Page getCurrentPage() {
		return null;
	}

    /**
     * @return {@link WCMBindings#RESOURCE_PAGE} binding if available, otherwise null is returned
     * */
    public final Page getResourcePage() {
		return null;
	}

    /**
     * @return {@link WCMBindings#PAGE_PROPERTIES} binding if available, otherwise null is returned
     * */
    public final ValueMap getPageProperties() {
		return null;
	}

    /**
     * @return {@link WCMBindings#PROPERTIES} binding if available, otherwise null is returned
     * */
    public final ValueMap getProperties() {
		return null;
	}

    /**
     * @return {@link WCMBindings#DESIGNER} binding if available, otherwise null is returned
     * */
    public final Designer getDesigner() {
		return null;
	}

    /**
     * @return {@link WCMBindings#CURRENT_DESIGN} binding if available, otherwise null is returned
     * */
    public final Design getCurrentDesign() {
		return null;
	}

    /**
     * @return {@link WCMBindings#CURRENT_STYLE} binding if available, otherwise null is returned
     * */
    public final Style getCurrentStyle() {
		return null;
	}

    /**
     * @return {@link WCMBindings#COMPONENT} binding if available, otherwise null is returned
     * */
    public final Component getComponent() {
		return null;
	}

    /**
     * @return {@link WCMBindings#INHERITED_PAGE_PROPERTIES} binding if available, otherwise null is returned
     * */
    public final ValueMap getInheritedProperties() {
		return null;
	}

    /**
     * @return Sling {@link SlingBindings#RESOURCE} binding if available, otherwise null is returned
     * */
    public final Resource getResource() {
		return null;
	}

    /**
     * @return ResourceResolver associated with the current request
     * */
    public final ResourceResolver getResourceResolver() {
		return null;
	}

    /**
     * @return Sling {@link SlingBindings#REQUEST} binding if available, otherwise null is returned
     * */
    public final SlingHttpServletRequest getRequest() {
		return null;
	}

    /**
     * @return Sling {@link SlingBindings#RESPONSE} binding if available, otherwise null is returned
     * */
    public final SlingHttpServletResponse getResponse() {
		return null;
	}

    /**
     * @return sling script helper binding {@link SlingBindings#SLING} if available, otherwise null is returned
     * */
    public final SlingScriptHelper getSlingScriptHelper() {
		return null;
	}
}
