/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.comments.api;

import javax.annotation.Nonnull;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;

import com.adobe.cq.social.scf.CollectionPagination;
import com.fasterxml.jackson.annotation.JsonProperty;

/**
 * Base implementation of the {@link CommentCollectionConfiguration}.
 */
public class AbstractCommentCollectionConfiguration implements CommentCollectionConfiguration {

    private boolean allowsVoting;
    private boolean isRTEEnabled;
    private boolean allowsAttachment;
    private final boolean allowsTranslateAllButton;
    private final boolean allowsFlagging;
    private final boolean allowsClose;
    private final boolean allowsDeny;
    private final boolean allowsEdit;
    private final boolean allowsFollowing;
    private boolean allowsDelete;
    private boolean allowsCustomFlagReason = true;
    private int pageSize = 10;
    private final ValueMap configValueMap;

    /**
     * Constructor.
     * @param resource the target resource of the collection or comment
     * @param cs the {@link CommentSystem} instance of the CommentCollection
     */
    public AbstractCommentCollectionConfiguration(final Resource commentSystem) {
        this(ResourceUtil.getValueMap(commentSystem));
    }

    /**
     * Constructor.
     * @param valuemap properties valuemap to use for the configuration
     */
    public AbstractCommentCollectionConfiguration(@Nonnull final ValueMap vm) {
        allowsVoting =
            vm.get(CommentCollectionConfiguration.PN_ALLOW_VOTING,
                CommentCollectionConfiguration.PV_DEFAULT_ALLOW_VOTING);
        allowsEdit = true;// cs.allowsEdit();

        pageSize = vm.get(PN_MAX_ITEMS_PER_PAGE, CollectionPagination.DEFAULT_PAGE_SIZE);
        allowsTranslateAllButton = vm.get(PN_ALLOW_TRANSLATE_ALL, PV_DEFAULT_ALLOW_TRANSLATE_ALL);
        allowsCustomFlagReason = vm.get(PN_ALLOW_CUSTOM_FLAGREASON, PV_DEFAULT_ALLOW_CUSTOM_FLAGREASON);
        allowsAttachment = vm.get(PN_ALLOW_ATTACHMENT, PV_DEFAULT_ALLOW_ATTACHMENT);
        isRTEEnabled = vm.get(PN_RTE_ENABLED, PV_DEFAULT_RTE_ENABLED);
        allowsDeny = vm.get(PN_ALLOW_DENY, PV_DEFAULT_ALLOW_DENY);
        allowsFlagging = vm.get(PN_ALLOW_FLAGGING, PV_DEFAULT_ALLOW_FLAGGING);
        allowsFollowing = vm.get(PN_ALLOW_FOLLOWING, PV_DEFAULT_ALLOW_FOLLOWING);

        allowsClose = vm.get(PN_ALLOW_CLOSE, PV_DEFAULT_ALLOW_CLOSE);
        allowsDelete = vm.get(PN_ALLOW_DELETE_COMMENTS, PV_DEFAULT_ALLOW_DELETE_COMMENTS);
        this.configValueMap = vm;
    }

    @Override
    @JsonProperty("isVotingAllowed")
    public boolean isVotingAllowed() {
		return false;
	}

    @Override
    @JsonProperty("isAttachmentAllowed")
    public boolean isAttachmentAllowed() {
		return false;
	}

    @Override
    @JsonProperty("isRTEEnabled")
    public boolean isRTEEnabled() {
		return false;
	}

    @Override
    @JsonProperty("isCustomFlagReasonAllowed")
    public boolean isCustomFlagReasonAllowed() {
		return false;
	}

    @Override
    @JsonProperty("isTranslateAllButtonAllowed")
    public boolean isTranslateAllButtonAllowed() {
		return false;
	}

    @Override
    public int getPageSize() {
		return 0;
	}

    @Override
    @JsonProperty("isFlaggingAllowed")
    public boolean isFlaggingAllowed() {
		return false;
	}

    @Override
    @JsonProperty("isCloseAllowed")
    public boolean isCloseAllowed() {
		return false;
	}

    @Override
    @JsonProperty("isDenyAllowed")
    public boolean isDenyAllowed() {
		return false;
	}

    @Override
    @JsonProperty("isEditAllowed")
    public boolean isEditAllowed() {
		return false;
	}

    @Override
    @JsonProperty("isDeleteAllowed")
    public boolean isDeleteAllowed() {
		return false;
	}

    protected void setAllowsDelete(final boolean flag) {
	}

    protected void setAllowsAttachment(final boolean flag) {
	}

    protected void setAllowsVoting(final boolean flag) {
	}

    protected void setRTEEnabled(final boolean flag) {
	}

    /**
     * Gets the configuration value map that this configuration object is based off of.
     * @return a value map that has configuration details in it.
     */
    @Nonnull
    protected ValueMap getConfigValueMap() {
		return null;
	}

    @Override
    public boolean isFollowingAllowed() {
		return false;
	}

}
