/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.commons.comments.api;

import com.fasterxml.jackson.annotation.JsonProperty;

public interface CommentCollectionConfiguration {
    /**
     * The property name for allowing users to vote.
     */
    String PN_ALLOW_VOTING = "allowVoting";

    /**
     * The default value of Allow Voting property
     */
    boolean PV_DEFAULT_ALLOW_VOTING = false;

    /**
     * The property name of the allowing users voting root.
     */
    String PN_VOTING_ROOT = "votingRoot";

    String PN_ALLOW_TAGGING = "allowTagging";

    /**
     * The default value of allow users voting root.
     */
    String PV_DEFAULT_VOTING_ROOT = "voting";

    /**
     * The property name of the voting type.
     */
    String PN_VOTING_TYPE = "votingType";

    /**
     * The default value of allow users voting type.
     */
    String PV_DEFAULT_VOTING_TYPE = "social/tally/components/hbs/voting";

    /**
     * The property name for allowing users or moderators to delete comments.
     */
    String PN_ALLOW_DELETE_COMMENTS = "allowDeleteComments";

    /**
     * The default property value for allowing users or moderators to delete comments.
     */
    boolean PV_DEFAULT_ALLOW_DELETE_COMMENTS = false;

    /**
     * The property name for allowing users to flag comments.
     */
    String PN_ALLOW_FLAGGING = "allowFlagComments";

    /**
     * The default property value for allowing users to flag comments.
     */
    boolean PV_DEFAULT_ALLOW_FLAGGING = false;

    /**
     * The property name for allowing users to close comments.
     */
    String PN_ALLOW_CLOSE = "allowCloseComments";

    /**
     * The default property value for allowing users to close comments.
     */
    boolean PV_DEFAULT_ALLOW_CLOSE = false;

    /**
     * The property name for allowing users to deny comments.
     */
    String PN_ALLOW_DENY = "allowDenyComments";

    /**
     * The default property value for allowing users to deny comments.
     */
    boolean PV_DEFAULT_ALLOW_DENY = false;

    /**
     * The property name for enabling the rich text editor.
     */
    String PN_RTE_ENABLED = "rteEnabled";

    /**
     * The default state of the rich text editor.
     */
    boolean PV_DEFAULT_RTE_ENABLED = false;

    /**
     * The property name for allowing users to attach files.
     */
    String PN_ALLOW_ATTACHMENT = "allowFileUploads";

    /**
     * The default property value for allowing users to attach files.
     */
    boolean PV_DEFAULT_ALLOW_ATTACHMENT = false;

    /**
     * The property name for allowing users to use a typed in flag reason.
     */
    String PN_ALLOW_CUSTOM_FLAGREASON = "allowCustomFlagReason";

    /**
     * The default property value for allowing a custom flag reason.
     */
    boolean PV_DEFAULT_ALLOW_CUSTOM_FLAGREASON = false;

    /**
     * The property name for allowing users to translate a whole thread.
     */
    String PN_ALLOW_TRANSLATE_ALL = "allowTranslateAllButton";

    /**
     * The default property value for allowing user to translate a whole thread.
     */
    boolean PV_DEFAULT_ALLOW_TRANSLATE_ALL = false;

    /**
     * The property name for the maximum number of comments on a page.
     */
    String PN_MAX_ITEMS_PER_PAGE = "maxPerPage";

    /**
     * The property name for the allow Following.
     */
    String PN_ALLOW_FOLLOWING = "allowFollowing";

    /**
     * The default property value for allowing following.
     */
    boolean PV_DEFAULT_ALLOW_FOLLOWING = false;

    /**
     * @return true if the comment can be voted
     */
    @JsonProperty("isVotingAllowed")
    boolean isVotingAllowed() {
		return false;
	}

    /**
     * Allows attachment.
     * @return true, if successful
     */
    @JsonProperty("isAttachmentAllowed")
    boolean isAttachmentAllowed() {
		return false;
	}

    /**
     * Checks if is RTE enabled.
     * @return true, if is RTE enabled
     */
    @JsonProperty("isRTEEnabled")
    boolean isRTEEnabled() {
		return false;
	}

    /**
     * @return boolean indicating if custom flag reasons are allowed. The JCR of this attribute is called
     *         "seFlagReasonFreeformText"
     */
    @JsonProperty("isCustomFlagReasonAllowed")
    boolean isCustomFlagReasonAllowed() {
		return false;
	}

    /**
     * @return boolean indicating if translate all button are allowed. The JCR of this attribute is called
     *         "allowTranslateAllButton"
     */
    @JsonProperty("isTranslateAllButtonAllowed")
    boolean isTranslateAllButtonAllowed() {
		return false;
	}

    /**
     * @return boolean indicating if closing a tree of replies is allowed.The JCR of this attribute is called
     *         "allowCloseComments"
     */
    boolean isCloseAllowed() {
		return false;
	}

    /**
     * @return boolean indicating if flagging content is allowed.The JCR of this attribute is called
     *         "allowFlagComments"
     */
    boolean isFlaggingAllowed() {
		return false;
	}

    /**
     * @return boolean indicating if denying a tree of replies is allowed.The JCR of this attribute is called
     *         "allowDenyComments"
     */
    boolean isDenyAllowed() {
		return false;
	}

    /**
     * @return boolean indicating if denying a tree of replies is allowed. The JCR of this attribute is called
     *         "allowDenyComments"
     */
    boolean isEditAllowed() {
		return false;
	}

    /**
     * @return boolean indicating if deleting comments is allow for a user. The JCR of this attribute is called
     *         "allowDeleteComments"
     */
    boolean isDeleteAllowed() {
		return false;
	}

    /**
     * @return the default number of comments on a page as configured on the component. If there is no configuration,
     *         this defaults to 10.
     */
    int getPageSize() {
		return 0;
	}

    /**
     * @return setting value of the following allowed configuration.
     */
    boolean isFollowingAllowed() {
		return false;
	}
}
