/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.forum.client.api;

import org.apache.sling.api.resource.Resource;

import com.adobe.cq.social.commons.comments.api.Comment;
import com.adobe.cq.social.commons.comments.api.CommentCollectionConfiguration;
import com.adobe.cq.social.commons.comments.api.PageInfo;
import com.adobe.cq.social.scf.User;

/**
 * A Post represents a topic of a forum, or a post of a topics. The {@link Post.isTopic()} method indicates whether an
 * instance of a Post is a topic or not.
 */
public interface Post<T extends CommentCollectionConfiguration> extends Comment<T> {
    /**
     * JCR properties that maintains the id of the user that effected the last modification of this topic.
     */
    final String LAST_SELF_MOD_BY = "cq:lastSelfModifiedBy";
    final String PN_SUBJECT = "jcr:title";
    final String POST_POSTFIX = "_tosp";

    /**
     * This method returns the forum post's subject as entered by the user. In general only top-level posts (topics)
     * have a subject. If the post doesn't have a subject, the subject of top level parent (topic) is returned.
     * @return A <code>String</code> representing the post's subject or <code>null</code> if not defined.
     */
    String getSubject() {
		return null;
	}

    /**
     * Returns the user that effected the last modification of this post.
     * @return A <code>User</code> last modified this post.
     */
    User getLastModifiedUser() {
		return null;
	}

    /**
     * Indicates whether this post is pinned. A post is pinned if it is a topic (top level post) or its parent topic
     * is pinned.
     * @return <code>true</code> if this post is pinned.
     */
    boolean isPinned() {
		return false;
	}

    /**
     * Indicates whether this forum or topic is closed for further addition of topics/posts.
     * @return
     */
    @Override
    boolean isClosed() {
		return false;
	}

    /**
     * Check if this instance is a Post or not.
     * @return <code>true</code> if this post is a topic.
     */
    boolean isTopic() {
		return false;
	}

    /**
     * Get the forum id for this post.
     * @return the forum id.
     */
    String getForumId() {
		return null;
	}

    /**
     * Returns the latest post, i.e. the post with the most recent modification date. If this post is not a topic or
     * doesn't have any posts, <code>null</code> is returned.
     * @return The latest post of this topic or <code>null</code> if not found.
     */
    String getLatestPost() {
		return null;
	}

    /**
     * @return a search engine friendly url to render this post if this post is a topic. if the post is a topic, this
     *         is blank.
     */
    String getFriendlyUrl() {
		return null;
	}

    /**
     * Set latest post.
     * @param post the latest post.
     */
    void setLatestPost(final Resource post) {
	}

    /**
     * Set the number of replies.
     * @param replies the latest number of replies.
     */
    void setNumReplies(final int replies) {
	}

    /**
     * Gets information about the pages for this collection. This can be used by the page block helper to easily
     * render various pagination UIs.
     * @return information about the pagination system
     */
    @Override
    PageInfo getPageInfo() {
		return null;
	}

    /**
     * @return true if the topic is closed.
     */
    boolean isTopicClosed() {
		return false;
	}

    /**
     * @return true if the entire forum is closed.
     */
    boolean isForumClosed() {
		return false;
	}
}
