/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.scf.core.operations;

import java.lang.reflect.AnnotatedElement;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.apache.felix.scr.annotations.Component;
import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.request.RequestPathInfo;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.wrappers.SlingHttpServletRequestWrapper;
import org.apache.sling.api.wrappers.SlingRequestPaths;
import org.apache.sling.commons.json.JSONException;
import org.apache.sling.commons.json.JSONObject;
import org.apache.sling.servlets.post.JSONResponse;
import org.apache.sling.servlets.post.PostOperation;
import org.apache.sling.servlets.post.PostResponse;
import org.apache.sling.servlets.post.SlingPostConstants;
import org.apache.sling.servlets.post.SlingPostProcessor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.commons.annotation.Endpoint;
import com.adobe.cq.social.commons.annotation.Parameters;
import com.adobe.cq.social.scf.JsonException;
import com.adobe.cq.social.scf.OperationException;
import com.adobe.cq.social.scf.SocialOperationInfo;
import com.adobe.cq.social.scf.SocialOperationResult;
import com.adobe.cq.social.ugcbase.core.SocialResourceUtils;

/**
 * Abstract class that implements PostOperation and provides utility methods to send correct HTTP responses. This
 * class can be extended to implement PostOperations for components.
 */
@Component(metatype = false, componentAbstract = true)
public abstract class AbstractSocialOperation implements PostOperation, SocialOperationInfo {

    private static final Logger LOG = LoggerFactory.getLogger(AbstractSocialOperation.class);

    @Override
    public final void run(final SlingHttpServletRequest request, final PostResponse response,
        final SlingPostProcessor[] processors) {
		}

    /**
     * Default implementation of {@link SocialOperationInfo} interface. {@inheritDoc}
     */
    @Override
    public final Endpoint getEndpoint() {
		return null;
	}

    /**
     * Default implementation of {@link SocialOperationInfo} interface. {@inheritDoc}
     */
    @Override
    public final Parameters getRequestParameters() {
		return null;
	}

    /**
     * @param e an exception thrown by the operation that defines the HTTP response status and message.
     * @param request the request that triggered the operation
     * @param response the HTTP response for the request.
     */
    protected void sendResponse(final OperationException e, final SlingHttpServletRequest request,
        final PostResponse response) {
		}

    /**
     * @param result the result returned after performing the operation.
     * @param request the request that triggered the operation
     * @param response the HTTP response for the request.
     */
    protected void sendResponse(final SocialOperationResult result, final SlingHttpServletRequest request,
        final PostResponse response) {
		}

    /**
     * @param request the request that triggered the operation.
     * @param path path to resource that was touched by the operation.
     * @return the externalized URL to the resource.
     */
    protected final String externalizePath(final SlingHttpServletRequest request, final String path) {
		return null;
	}

    /**
     * Override this method to implement the operation and return appropriate results.
     * @param request the requested that triggered the operation
     * @return a SocialOperationResult that contains the result of the operation.
     * @throws OperationException when an error occurs
     */
    protected abstract SocialOperationResult performOperation(SlingHttpServletRequest request)
        throws OperationException {
		return null;
	}

    class NonExistingResourceWrappingRequest extends SlingHttpServletRequestWrapper {
        final private Resource newResource;

        public NonExistingResourceWrappingRequest(final SlingHttpServletRequest wrappedRequest,
            final Resource resource) {
            super(wrappedRequest);
            this.newResource = resource;
        }

        @Override
        public Resource getResource() {
			return null;
		}
    }
}
