/**************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.scoring.api;

import java.util.List;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;

/**
 * Scoring service exposes operations like getPoints, resolveSegment needed by components.
 */
public interface ScoringService {

    /**
     * Retrieves default score of user specified via scoring osgi configuration.
     * @param resourceResolver - resourceResolver of the user whose score needs to be retrieved
     * @return score of the user
     * @throws ScoringException thrown if unable to retrieve the score
     */
    long getPoints(ResourceResolver resourceResolver) throws ScoringException {
		return 0L;
	}

    /**
     * Retrieves score specified by scoreName for user.
     * @param resourceResolver - resourceResolver of the user whose score needs to be retrieved
     * @param scoreName - name of score to be retrieved
     * @return score of the user
     * @throws ScoringException thrown if unable to retrieve the score
     */
    long getPoints(ResourceResolver resourceResolver, String scoreName) throws ScoringException {
		return 0L;
	}

    /**
     * Resolves a scoring segment in current path.
     * @param segmentPath - path of segment to be resolved
     * @param userId - userId to be used for resolving the segment
     * @return result of resolution
     * @throws ScoringException thrown if unable to resolve the segment
     */
    boolean resolveScoringSegment(String segmentPath, String userId) throws ScoringException {
		return false;
	}

    /**
     * Retrieves all segment scoring for a user.
     * @param userResource the user's resource
     * @return list of scores
     */
    List<UserScore> getUserScores(Resource userResource) {
		return null;
	}

}
