/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *************************************************************************/
package com.adobe.granite.ocs.api;


import aQute.bnd.annotation.ProviderType;
import com.adobe.granite.ocs.api.provider.Provider;
import org.apache.sling.api.resource.Resource;

import java.util.List;

/**
 * The OCS deployment.
 *
 * An OCS deployment can have multiple {@link com.adobe.granite.ocs.api.Environment environments} and acts as a project
 * container. Each deployment has exactly one {@link com.adobe.granite.ocs.api.provider.Provider provider} assigned,
 * giving the context to what provider a deployment can be deployed to.
 *
 * If you have access to an OCS deployment resource, the resource can be adapted to a deployment.
 */
@ProviderType
public interface Deployment extends Resource {

    /**
     * Gives access to the deployment title.
     *
     * @return The deployment title.
     */
    public String getTitle() {
		return null;
	}

    /**
     * Gives access to the deployment description.
     *
     * @return The deployment description.
     */
    public String getDescription() {
		return null;
	}

    /**
     * Gives access to the OCS tenant this deployment belongs to.
     *
     * @return The {@link com.adobe.granite.ocs.api.Tenant tenant} of this deployment.
     */
    public Tenant getTenant() {
		return null;
	}

    /**
     * Get a listing of all the environments of this deployment.
     *
     * @return List of {@link com.adobe.granite.ocs.api.Environment environments} or an empty list if this deployment
     * has no environments.
     */
    public List<Environment> getEnvironments() {
		return null;
	}

    /**
     * Get a specific environment according to the environment name.
     *
     * @param environmentName The name of the environment to get.
     *
     * @return The {@link com.adobe.granite.ocs.api.Environment environment} or <code>null</code> if there is no
     * environment with the given name.
     */
    public Environment getEnvironment(String environmentName) {
		return null;
	}

    /**
     * Get the provider associated to this deployment.
     *
     * @return The {@link com.adobe.granite.ocs.api.provider.Provider provider} of this deployment or <code>null</code>
     * if this deployment does not have a provider.
     */
    public Provider getProvider() {
		return null;
	}

    /**
     * Deployment properties.
     */
    public enum Properties {
        /**
         * The server admin email.
         */
        SERVER_ADMIN_EMAIL("server_admin_email");

        private String propertyName = null;

        Properties(String propertyName) {
            this.propertyName = propertyName;
        }

        public String propertyName() {
            return this.propertyName;
        }
    }
}
