/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *************************************************************************/
package com.adobe.granite.ocs.api;

import aQute.bnd.annotation.ProviderType;
import org.apache.sling.api.resource.Resource;

import java.util.List;

/**
 * The OCS environment.
 *
 * An OCS environment is associated to an OCS {@link com.adobe.granite.ocs.api.Deployment deployment} and can have
 * multiple OCS {@link com.adobe.granite.ocs.api.Host hosts}.
 *
 * If you have access to an OCS environment resource, the resource can be adapted to an environment.
 */
@ProviderType
public interface Environment extends Resource {


    /**
     * Gives access to the environment title.
     *
     * @return The environment title.
     */
    public String getTitle() {
		return null;
	}

    /**
     * Gives access to the environment description.
     *
     * @return The environment description.
     */
    public String getDescription() {
		return null;
	}

    /**
     * Get a listing of the hosts assigned to this environment.
     *
     * @return List of {@link com.adobe.granite.ocs.api.Host hosts} or empty list if this environment has no hosts.
     */
    public List<Host> getHosts() {
		return null;
	}

    /**
     * Get the host according to the given host name.
     *
     * @param hostName The name of the host to get.
     *
     * @return The {@link com.adobe.granite.ocs.api.Host host} or <code>null</code> if the environment has no host with
     *         the given name.
     */
    public Host getHost(String hostName) {
		return null;
	}

    /**
     * Gives access to the OCS deployment this environment belongs to.
     *
     * @return The {@link com.adobe.granite.ocs.api.Deployment deployment} this environment belongs to.
     */
    public Deployment getDeployment() {
		return null;
	}

    /**
     * Gives access to the status information of this environment.
     *
     * @return {@link com.adobe.granite.ocs.api.EnvironmentStatus status} of this environment.
     */
    public EnvironmentStatus getStatus() {
		return null;
	}

    /**
     * Environment properties.
     */
    public enum Properties {

        DEPLOYMENT_ID("deploymentId");

        private String propertyName = null;

        Properties(String propertyName) {
            this.propertyName = propertyName;
        }

        public String propertyName() {
            return this.propertyName;
        }
    }
}
