/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *************************************************************************/
package com.adobe.granite.ocs.api.provider;

import aQute.bnd.annotation.ProviderType;
import com.adobe.granite.ocs.api.Deployment;
import org.apache.sling.api.resource.Resource;

import java.util.Set;

/**
 * The OCS provider.
 * <p/>
 * The OCS provider is an a abstraction for tasks related to a specific cloud provider.
 */
@ProviderType
public interface Provider extends Resource {

    /**
     * Give access to the provider title.
     *
     * @return The provider title.
     */
    String getTitle() {
		return null;
	}

    /**
     * Gives access to the provider description.
     *
     * @return The description of the provider.
     */
    String getDescription() {
		return null;
	}

    /**
     * Gives access to the identity to use for this cloud provider.
     * <p/>
     * The meaning of identity depends on the implementation. For AWS for example, it means the API key, for other
     * providers it could as well be a user id.
     *
     * @return This providers identity.
     */
    String getIdentity() {
		return null;
	}

    /**
     * Gives access to the secret to use for this cloud provider.
     * <p/>
     * The meaning of the secret depends on the implementation. For AWS for example, it means the API secret, for other
     * providers it could as well be a password.
     *
     * @return
     */
    String getSecret() {
		return null;
	}

    /**
     * Gives access to the deployment this provider belongs to.
     *
     * @return The {@link com.adobe.granite.ocs.api.Deployment deployment} this provider belongs to.
     */
    Deployment getDeployment() {
		return null;
	}

    /**
     * Creates the given environment in the cloud as per implementation.
     * <p/>
     * Implementations should keep this an asynchronous call and just start the creation and not block until the
     * creation is complete.
     *
     * @param environmentName The environment to create in the cloud.
     *
     * @return The status as per {@link com.adobe.granite.ocs.api.provider.ProviderStatus.Status}.
     */
    ProviderStatus createEnvironment(String environmentName) {
		return null;
	}

    /**
     * Deletes the given environment in the cloud as per implementation.
     * <p/>
     * Implementations should keep this an asynchronous call and just start the deletion and not block until the
     * deletion is complete.
     *
     * @param environmentName The environment to delete in the cloud.
     *
     * @return The status as per {@link com.adobe.granite.ocs.api.provider.ProviderStatus.Status}.
     */
    ProviderStatus deleteEnvironment(String environmentName) {
		return null;
	}

    /**
     * Check the status of the given environment in the cloud.
     *
     * @return The status as per {@link com.adobe.granite.ocs.api.provider.ProviderStatus.Status}.
     */
    ProviderStatus getStatus(String environmentName) {
		return null;
	}

    /**
     * Refreshes the given environment with with information from the cloud.
     *
     * @param environmentName The environment to refresh.
     */
    void refresh(String environmentName) {
	}

    /**
     * Checks if the cloud resources of the given environment are ready.
     *
     * @param environmentName The environment to check for readiness.
     *
     * @return <code>true</code> if the environment's cloud resources are ready, <code>false</code> otherwise.
     */
    public boolean isReady(String environmentName) {
		return false;
	}

    /**
     * Registers a new RSA public key for the given environment under the given name.
     *
     * @param environmentName The name of the environment for which to register the key.
     * @param name            The name of the public key.
     * @param publicKey       The Base64 encoded RSA key material.
     *
     * @return The status of the operation.
     */
    ProviderStatus registerPublicKey(String environmentName, String name, String publicKey) {
		return null;
	}

    /**
     * Unregisters a RSA public key from the given environment.
     *
     * @param environmentName The name of the environment for which to unregister the key.
     * @param keyName            The name of the public key to unregister.
     *
     * @return The status of the operation.
     */
    ProviderStatus unregisterKey(String environmentName, String keyName) {
		return null;
	}

    /**
     * Retrieves the set of key pair names currently known to this provider.
     *
     * @return The set with the key pair names.
     */
    Set<String> getKeyPairNames() {
		return null;
	}

    /**
     * Provider properties common to all provider implementations.
     */
    enum Property {
        /**
         * The identity property common to all providers.
         */
        IDENTITY("providerIdentity"),

        /**
         * The secret property common to all providers.
         */
        SECRET("providerSecret");

        private String propertyName = null;

        Property(String propertyName) {
            this.propertyName = propertyName;
        }

        public String propertyName() {
            return this.propertyName;
        }
    }


}