/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 *************************************************************************/
package com.adobe.granite.ocs.api.provider;

import aQute.bnd.annotation.ProviderType;
import org.apache.sling.api.resource.Resource;

/**
 * Factory service for provider factories.
 *
 * Each provider implementation must register its own factory for creating specific provider instances. Implementations
 * must register using the {@value #PROVIDER_ID} property.
 *
 * Clients are not expected to implement this interface.
 */
@ProviderType
public interface ProviderFactory {

    /**
     * Service property name to use for registering new provider factories.
     */
    public static final String PROVIDER_ID = "granite.ocs.provider.id";

    /**
     * Create a provider instance based on the given provider resource.
     *
     * @param providerResource The provider resource to use.
     *
     * @return The provider instance.
     */
    public Provider createProvider(Resource providerResource) {
		return null;
	}

}
