/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2011 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.security.user;

import java.util.Iterator;

import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.apache.jackrabbit.api.security.user.Authorizable;
import org.apache.jackrabbit.api.security.user.Group;

import aQute.bnd.annotation.ProviderType;

/**
 * The <code>UserPropertiesManager</code> interface provides access to
 * <code>UserProperties</code> associated with a given authorizable.
 */
@ProviderType
public interface UserPropertiesManager {

    /**
     * Create a new instance of <code>UserProperties</code>. Please note that
     * {@link javax.jcr.Session#save()} must be called by the editing JCR
     * session in order to persist the new user properties.
     *
     * @param authorizableId The ID of the associated authorizable.
     * @param relPath A name or relative path identifying the new user properties.
     * @return A new <code>UserProperties</code> instance. Note that the
     * associated JCR node will be {@link javax.jcr.Item#isNew() NEW} until
     * {@link javax.jcr.Session#save() Session.save} is called to persist the changes.
     * @throws javax.jcr.RepositoryException If an error occurs or the specified
     * relative path is invalid.
     */
    UserProperties createUserProperties(String authorizableId, String relPath) throws RepositoryException {
		return null;
	}

    /**
     * Retrieve the user properties identified by the given relative path and
     * the specified authorizable ID.
     *
     * @param authorizableId Identifier of the target authorizable.
     * @param relPath A name or relative path identifying the user properties.
     * @return An instance of <code>UserProperties</code> or <code>null</code>
     * if the authorizable does not exist or it has no user properties at the
     * specified relative path.
     * @throws RepositoryException If an error occurs or the specified
     * relative path is invalid.
     */
    UserProperties getUserProperties(String authorizableId, String relPath) throws RepositoryException {
		return null;
	}

    /**
     * Retrieve the user properties identified by the given relative path and
     * the specified authorizable.
     *
     * @param authorizable The target authorizable.
     * @param relPath A name or relative path identifying the user properties.
     * @return An instance of <code>UserProperties</code> or <code>null</code>
     * if the authorizable has no user properties at the specified relative path.
     * @throws RepositoryException If an error occurs or the specified
     * relative path is invalid.
     */
    UserProperties getUserProperties(Authorizable authorizable, String relPath) throws RepositoryException {
		return null;
	}

    /**
     * Retrieve the user properties identified by the given node.
     *
     * @param userPropertiesNode The node representing the user properties.
     * @return The user properties identified by the given node.
     * @throws RepositoryException If an exception occurs or if the given node
     * does not represent valid user properties (e.g. not associated with an
     * authorizable).
     */
    UserProperties getUserProperties(Node userPropertiesNode) throws RepositoryException {
		return null;
	}

    /**
     * Retrieves a composite (aggregation) of {@link UserProperties} associated with the authorizable identified by the
     * given <code>authorizableId</code> and corresponding to  and in the order of the given <code>relPaths</code>. The
     * ordering is relevant for e.g. accessing a property found on multiple user property nodes, in which case it is
     * inherited according to the order (last has precedence).
     *
     * @param authorizableId The ID of the {@link Authorizable} for which to retrieve the user properties composite.
     * @param relPaths       An array of strings denoting the desired relative paths of the user property nodes and
     *                       their order in which to be included in the composite.
     *
     * @return A composite of {@link UserProperties}.
     *
     * @throws RepositoryException If an error occurs during repository access or no authorizable was found with the
     *                             given <code>authorizableId</code>.
     */
    UserPropertiesComposite getUserPropertiesComposite(String authorizableId, String[] relPaths)
            throws RepositoryException {
		return null;
	}

    /**
     * Retrieves a composite (aggregation) of {@link UserProperties} associated with the authorizable identified by the
     * given <code>authorizableId</code> and included by the given <code>filter</code>.
     *
     * @param authorizableId The ID of the {@link Authorizable} for which to retrieve the user properties composite.
     * @param filter         A {@link UserPropertiesFilter} by which user properties are included or excluded.
     *
     * @return A {@link UserPropertiesComposite} containing the {@link UserProperties} included by the filter.
     *
     * @throws RepositoryException If an error occurs during repository access or no authorizable was found with the
     *                             given <code>authorizableId</code>.
     */
    UserPropertiesComposite getUserPropertiesComposite(String authorizableId, UserPropertiesFilter filter)
            throws RepositoryException {
		return null;
	}

    /**
     * Returns an iterator over the properties of all groups that the authorizable with <code>authorizableId</code> is
     * a member of. please note that only groups are included of which the underlying session has read access to.
     *
     * @param authorizableId the id of the authorizable
     * @param relPath the relative path to build the user properties from
     * @param declaredOnly if {@code true} only declared groups are included
     * @return an iterator of user properties
     * @throws RepositoryException if an error occurrs
     *
     * @see org.apache.jackrabbit.api.security.user.Authorizable#memberOf()
     * @see org.apache.jackrabbit.api.security.user.Authorizable#declaredMemberOf()
     */
    Iterator<UserProperties> getMemberOfUserProperties(String authorizableId, String relPath, boolean declaredOnly)
            throws RepositoryException {
		return null;
	}

    /**
     * Returns an iterator over the user properties of all members of the given group.
     *
     * @param group the group to retrieve the members from
     * @param relPath the relative path to build the user properties from
     * @param declaredOnly if {@code true} only delcated members are included
     * @return an iterator of user properties
     * @throws RepositoryException if an error occurrs
     *
     * @see org.apache.jackrabbit.api.security.user.Group#getMembers()
     * @see org.apache.jackrabbit.api.security.user.Group#getDeclaredMembers()
     */
    Iterator<UserProperties> getMemberUserProperties(Group group, String relPath, boolean declaredOnly)
            throws RepositoryException {
		return null;
	}

}
