/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.taskmanagement;

import com.adobe.granite.workflow.exec.InboxItem;
import com.adobe.granite.workflow.exec.Status;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

/**
 * Task class describes a task instance.
 */
public interface Task extends InboxItem {

    /**
     * Default task type.
     */
    String DEFAULT_TASK_TYPE = "default";

    /**
     * Gets the ID of the task.
     * @return String containing the id of the task.
     */
    String getId() {
		return null;
	}

    /**
     * Returns the ID of the parent task.
     * @return the ID of the parent task, or null if this is not a sub-task.
     */
    String getParentId() {
		return null;
	}

    /**
     * Gets the name of the task.
     * @return String containing the name of the task.
     */
    String getName() {
		return null;
	}

    /**
     * Sets the name of the task.
     * @param name String containing the new name of the task.
     */
    void setName(String name) {
	}

    /**
     * Gets the description of the task.
     * @return String containing the description of the task.
     */
    String getDescription() {
		return null;
	}

    /**
     * Sets the description of the task.
     * @param description String containing the new description of the task.
     */
    void setDescription(String description) {
	}

    /**
     * Gets the current status of the task.
     * @return Status instance representing the current task status.
     */
    Status getStatus() {
		return null;
	}

    /**
     * Gets the id of the owner of the task.
     * @return String containing the id of the owner of the task.
     */
    String getCurrentAssignee() {
		return null;
	}

    /**
     * Sets the id of the owner of the task.
     * @param ownerId String containing the id of the owner of the task.
     */
    void setCurrentAssignee(String ownerId) {
	}

    /**
     * Gets the type of this task.
     * @return String containing the name of the type of this task.
     */
    String getTaskTypeName() {
		return null;
	}

    /**
     * Gets the time this task was created.
     * @return Date containing the time this task was created.
     */
    Date getTimeStarted() {
		return null;
	}

    /**
     * Gets the ID for the user who created the task
     * @return String containing the create user's ID
     */
    String getCreatedBy() {
		return null;
	}

    /**
     * Gets the time this task was last updated.
     * @return Date containing the time this task was last updated.
     */
    Date getLastModified() {
		return null;
	}

    /**
     * Gets the ID for the user who last modified the task
     * @return String containing the user's ID
     */
    String getLastModifiedBy() {
		return null;
	}

    /**
     * Gets the time this task was last completed.
     * @return Date containing the time this task was completed.
     */
    Date getTimeEnded() {
		return null;
	}

    /**
     * Gets the ID for the user who completed the task
     * @return String containing the complete user's ID
     */
    String getCompletedBy() {
		return null;
	}

    /**
     * Sets the property.
     * @param name Name of the property to set.
     * @param value value of named property.
     */
    void setProperty(String name, Object value) {
	}

    /**
     * Gets a property with the given name.
     * @param aPropertyName Name of the property to retrieve.
     * @return value of the specified property.
     */
    Object getProperty(String aPropertyName) {
		return null;
	}

    /**
     * Gets an iterator of the property names for this task.
     * @return an task property name iterator.
     */
    Iterator<String> getPropertyNames() {
		return null;
	}

    /**
     * Sets the instructions for this task.
     * @param aInstructions String containing the new instructions for this task.
     */
    void setInstructions(String aInstructions) {
	}

    /**
     * Gets the instructions for this task.
     * @return String containing the instructions for this task.
     */
    String getInstructions() {
		return null;
	}

    /**
     * returns the list of action objects for this task.
     * @return list of action objects
     */
    List<TaskAction> getActions() {
		return null;
	}

    /**
     * sets the list of action objects for this task.
     * @param aListOfActions list of action objects
     */
    void setActions(List<TaskAction> aListOfActions) {
	}

    /**
     * gets the selected action used to complete the task.
     * @return action TaskAction represented the action used to compete the task
     */
    TaskAction getSelectedAction() {
		return null;
	}

    /**
     * Returns the list of subtasks for this task
     * @return the list of subtasks
     */
    List<Task> getSubTasks() {
		return null;
	}

    /**
     * Indicates if this task object is fully populated with sub task information.
     * For performance reasons some task objects may not be fully populated with sub task data.
     * @return true if the sub tasks are loaded in this task object
     */
    boolean isSubTaskInfoLoaded() {
		return false;
	}

    /**
     * Returns whether or not there are sub-tasks under this task.
     * Task instances from the server (ie: it does not depend on sub-tasks being actually loaded)
     * @return true if this task has sub-tasks, false otherwise
     */
    boolean hasSubTasks() {
		return false;
	}

    /**
     * Returns a content path this task is associated with.
     * @return the associated content path
     */
    String getContentPath() {
		return null;
	}

    /**
     * Sets a content path that this task is associated with.
     * @param contentPath the new content path for this task
     */
    void setContentPath(String contentPath) {
	}
}
