/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.granite.translation.api;

import java.io.InputStream;
import java.util.Iterator;

import com.adobe.granite.comments.Comment;
import com.adobe.granite.comments.CommentCollection;
import com.adobe.granite.translation.api.TranslationConstants.ContentType;

//
/** The Interface TranslationObject. */
public interface TranslationObject {

    /**
     * Gets the title of the TranslationObject. Title is auto-populated from TranslationObject.
     * @return the title
     */
    String getTitle() {
		return null;
	}

    /**
     * Gets the path to the Translation Object being translated. The path returned is to where the target Translation
     * Object is stored in AEM CRX de. The path is derived by coping that of the the source Translation Object.
     * @return the content url
     */
    String getTranslationObjectTargetPath() {
		return null;
	}

    /**
     * Gets the path to the source Translation Object. The path returned is to the where the source Translation Object
     * is stored in AEM CRX de.
     * @return the source content url
     */
    String getTranslationObjectSourcePath() {
		return null;
	}

    /**
     * Gets the version name of the source Object. If this name is null, then latest version of source content is
     * used.
     * @return the version name
     */
    String getSourceVersion() {
		return null;
	}

    /**
     * Gets the mime type of TranslationObject.
     * @return the mime type
     */
    String getMimeType() {
		return null;
	}

    /**
     * Gets confirmation return from the external translation service provider that they have received the
     * TranslationObject for translation. Translation service provider returns a unique TranslationObjectId.
     * @return Translation Object Id
     */
    String getId() {
		return null;
	}

    /**
     * Gets the comment collection. This returns all the comments added to the TranslationObject.
     * @return the comment Collection
     */
    CommentCollection<Comment> getCommentCollection() {
		return null;
	}

    /**
     * Gets the supporting translation objects iterator. Each TranslationObject can have a list of supporting
     * translation objects which add reference to the TranslationObject.
     * @return the supporting translation objects iterator
     */
    Iterator<TranslationObject> getSupportingTranslationObjectsIterator() {
		return null;
	}

    /**
     * Gets the supporting translation objects count.
     * @return the supporting translation objects count
     */
    int getSupportingTranslationObjectsCount() {
		return 0;
	}

    /**
     * Gets the Translation Job metadata
     * @return the Translation Job metadata
     */
    TranslationMetadata getTranslationJobMetadata() {
		return null;
	}

    /**
     * Gets the input stream for the Translation Object to be translated
     * @return the translation Object input stream
     */
    InputStream getTranslationObjectInputStream() throws TranslationException {
		return null;
	}

    /**
     * Gets the translated Object input stream. Returns null in case TranslationObject was not translated.
     * @return the translated Object input stream
     */
    InputStream getTranslatedObjectInputStream() {
		return null;
	}
}