/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2015 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.adobe.granite.ui.components;

import org.apache.commons.collections.CollectionUtils;
import org.apache.jackrabbit.util.ISO8601;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ValueMap;

import java.lang.reflect.Array;
import java.util.*;

/**
 * BulkEditValueMap is a ValueMap specific to the needs of Bulk Editing; it is aimed at merging the given Resources' ValueMaps.
 * Please not that the merge is actually "virtual" since under the hood the ValueMap will always be empty; in other words, <code>get(Object key)</code> performs an "on-demand" merge for the passed key.
 * In addition to providing the actual (merged) value for a given key, this specific ValueMap can also tell if a given key has a mixed value using <code>get(key + Field.IS_MIXED_SUFFIX)</code>.
 */
public class BulkEditValueMap implements ValueMap {

    private List<Resource> resources;
    private HashMap<String, MergedValue> cache; // Populated on-demand

    public BulkEditValueMap(List<Resource> resources) {
        this.resources = resources;
        this.cache = new HashMap<String, MergedValue>();
    }

    /**
     *  Retrieves the merged value for the passed key.
     *  Calling get(key + Field.IS_MIXED_SUFFIX) returns true if the value is mixed; false otherwise
     * @param key The key of the value to retrieve.
     * @return The merged value for the passed key; or a boolean telling if the value is mixed or not (if the key ends with Field.IS_MIXED_SUFFIX)
     */
    public Object get(Object key) {
		return null;
	}

    @SuppressWarnings("unchecked")
    public <T> T get(String name, Class<T> type) {
		return null;
	}

    @SuppressWarnings("unchecked")
    public <T> T get(String name, T defaultValue) {
		return null;
	}

    private Object getValue(String key) {
		return null;
	}

    // Key shouldn't contain the Field.IS_MIXED_SUFFIX
    private boolean isValueMixed(String key) {
		return false;
	}

    private MergedValue fetchMergedValue(String key) {
		return null;
	}

    // Returns all values found for the given key
    private List<Object> fetchValues(String key) {
		return null;
	}

    public boolean equals(Object o) {
		return false;
	}

    // ---------- Unsupported "Enumeration" methods
    // Note that the underlying map gets populated on-demand (wherever a value for a given key is requested)

    public int size() {
		return 0;
	}

    public boolean isEmpty() {
		return false;
	}

    public boolean containsKey(Object key) {
		return false;
	}

    public boolean containsValue(Object value) {
		return false;
	}

    public Set<String> keySet() {
		return null;
	}

    public Collection<Object> values() {
		return null;
	}

    public Set<Entry<String, Object>> entrySet() {
		return null;
	}

    public int hashCode() {
		return 0;
	}


    // ---------- Unsupported Modification methods

    public void clear() {
	}

    public Object put(String key, Object value) {
		return null;
	}

    public void putAll(Map<? extends String, ? extends Object> t) {
	}

    public Object remove(Object key) {
		return null;
	}


    // ---------- Type conversion helper
    @SuppressWarnings("unchecked")
    private <T> T convert(Object obj, Class<T> type) {
		return null;
	}

    private <T> T[] convertToArray(Object obj, Class<T> type) {
		return null;
	}

    // ---------- MergedValue inner class, contains the "merge" algorithm logic
    private class MergedValue {

        private Object value;
        private boolean isMixed;

        // Don't interfere with default values (null)
        private final static String MIXED_VALUE = "";

        public MergedValue(Object value, boolean isMixed) {
            this.value = value == null ? MIXED_VALUE : value;
            this.isMixed = isMixed;
        }

        public MergedValue(List<Object> values) {
            // Assuming the rest of the values are Arrays too
            if (values.get(0).getClass().isArray()) {
                doMergeArrayValues(values);
            } else {
                doMergeSingleValues(values);
            }
        }

        private void doMergeSingleValues(List<Object> values) {
		}

        private boolean equalsIgnoreOrder(Object[] first, Object[] second) {
			return false;
		}

        private void doMergeArrayValues(List<Object> arrayValues) {
		}

        public Object getValue() {
			return null;
		}

        public boolean isMixed() {
			return false;
		}
    }
}
