// =================================================================================================
// ADOBE SYSTEMS INCORPORATED
// Copyright 2006 Adobe Systems Incorporated
// All Rights Reserved
//
// NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the terms
// of the Adobe license agreement accompanying it.
// =================================================================================================

package com.adobe.internal.xmp.utils;

import java.io.IOException;
import java.io.Writer;
import java.util.HashSet;
import java.util.Stack;

import com.adobe.internal.xmp.impl.Utils;
import com.adobe.internal.xmp.options.SerializeOptions;


/**
 * Uncomplete implementation of XMPStreamWriter of the upcoming XML Streaming API,
 * that is adapted to be used for XMPCore.
 * Adds automatically correct closing of empty tags ("/>") and pretty printing.
 * Simplifies namespace handling; just tracks if a certain prefix has already been registered.
 * FfF: If StAX will become part of the JRE, it might be replaced.
 *
 * @author Stefan Makswit
 * @version $Revision$
 * @since 09.11.2006
 */
public class XMLStreamWriterImpl
{
	/** */
	private static final String DEFAULTNS = "";
	/** */
	private Writer writer;
	/** */
	private boolean startElementOpened = false;
	/** */
	private boolean emptyElement = false;
	/** these two stacks are used to implement the writeEndElement() method */
	private Stack qNameStack = new Stack();

	/** */
	private char[] newLineStr = new char[] { '\r' };
	/** */
	private int baseIndent = 0;
	/** */
	private char[] indentStr = new char[] { ' ', ' ' };
	/** */
	private int indentLevel = 0;
	/** Flag if chars enclosed in a tag are indented like body tags. */
	private boolean charIndent = false;
	/** flag if namespaces should start in a new line */
	private boolean namespaceLF = true;

	/** internal flag tracking the linefeeds */
	private boolean preventWhitespace = false;
	/** internal flag tracking the linefeeds */
	private boolean preventNextLF = true;
	/** stores the already registiered namspaces */
	private final HashSet registeredPrefixes = new HashSet();
	/** whitespaces (linefeeds) shall be escaped */
	private boolean escapeWhitespaces = true;


	/**
	 * Default constructor providing a writer the XML is written to
	 * and the serializsation options to determine indents, whitespaces and lineendings.
	 *
	 * @param writer a <code>Writer</code>
	 * @param options the serialization options
	 */
	public XMLStreamWriterImpl(Writer writer, SerializeOptions options)
	{
		this(writer);
		this.newLineStr = options.getNewline().toCharArray();
		this.indentStr = options.getIndent().toCharArray();
		this.baseIndent = options.getBaseIndent();
	}


	/**
	 * Default constructor providing a writer the XML is written to.
	 * @param writer a <code>Writer</code>
	 */
	public XMLStreamWriterImpl(Writer writer)
	{
		this.writer = writer;
	}


	/**
	 * Writes a start tag.
	 * @param qname a QName
	 * @throws IOException
	 */

	public void writeStartElement(String qname) throws IOException
	{
	}


	/**
	 * Differs from writeStartElement in that writeEndElement needs and must not be called.
	 * @param qname a QName
	 * @throws IOException
	 */
	public void writeEmptyElement(String qname) throws IOException
	{
	}


	/**
	 * Writes a closing tag if the tag in scope has body elements or "/>" if its
	 * empty.
	 *
	 * @throws IOException
	 */
	public void writeEndElement() throws IOException
	{
	}


	/**
	 * Closes the writer.
	 * @throws IOException
	 */
	public void close() throws IOException
	{
	}


	/**
	 * Calls flush() on the writer.
	 * @throws IOException
	 */
	public void flush() throws IOException
	{
	}


	/**
	 * Completes the document by writing the end tags for all open tags.
	 * @throws IOException
	 */
	public void writeEndDocument() throws IOException
	{
	}


	/**
	 * Writes an attribute into an open tag.
	 * @param qname a QName
	 * @param value the attribute value
	 * @throws IOException
	 */
	public void writeAttribute(String qname, String value) throws IOException
	{
	}


	/**
	 * Writes a namespace attribute and tracks if the namespace is already defined.
	 * @param prefix the namespace prefix.
	 * @param namespaceURI the namespace URI
	 * @throws IOException
	 */
	public void writeNamespace(String prefix, String namespaceURI) throws IOException
	{
	}


	/**
	 * Writes the default namespace
	 * @param namespaceURI the default namespace
	 * @throws IOException
	 */
	public void writeDefaultNamespace(String namespaceURI) throws IOException
	{
	}


	/**
	 * Writes a comment.
	 * @param comment
	 * @throws IOException
	 */
	public void writeComment(String comment) throws IOException
	{
	}


	/**
	 * Writes a processing instruction.
	 * @param target a target
	 * @throws IOException
	 */
	public void writeProcessingInstruction(String target) throws IOException
	{
	}


	/**
	 * Writes a processing instruction.
	 * @param target a target
	 * @param text the instruction text
	 * @throws IOException
	 */
	public void writeProcessingInstruction(String target, String text) throws IOException
	{
	}


	/**
	 * Writes the DTD.
	 * @param dtd
	 * @throws IOException
	 */
	public void writeDTD(String dtd) throws IOException
	{
	}


	/**
	 * Writes CData.
	 * @param data the data to write
	 * @throws IOException
	 */
	public void writeCData(String data) throws IOException
	{
	}


	/**
	 * Writes an entity reference.
	 * @param name the name of the reference
	 * @throws IOException
	 */
	public void writeEntityRef(String name) throws IOException
	{
	}


	/**
	 * Writes the XML-header and version.
	 * @throws IOException
	 */
	public void writeStartDocument() throws IOException
	{
	}


	/**
	 * Writes the XML-header and version.
	 * @param version th xml-version
	 * @throws IOException
	 */
	public void writeStartDocument(String version) throws IOException
	{
	}


	/**
	 * Writes the XML-header, encoding and version.
	 * @param encoding the file encoding
	 * @param version th XML-version
	 * @throws IOException
	 */
	public void writeStartDocument(String encoding, String version) throws IOException
	{
	}


	/**
	 * Writes XML-characters.
	 * @param text xml characters
	 * @throws IOException
	 */
	public void writeCharacters(String text) throws IOException
	{
	}


	/**
	 * Writes a part of a char array.
	 * @param buffer a character array
	 * @param off the offset within the array
	 * @param length the amount of character to write
	 * @throws IOException
	 */
	public void writeCharacters(char[] buffer, int off, int length) throws IOException
	{
	}


	/**
	 * @param escapeWhitespaces the escapeWhitespaces to set
	 */
	public void setEscapeWhitespaces(boolean escapeWhitespaces)
	{
	}


	/**
	 * Writes a string.
	 * @param str a String
	 * @throws IOException
	 */
	private void write(String str) throws IOException
	{
	}


	/**
	 * Writes a char
	 * @param ch a char
	 * @throws IOException
	 */
	private void write(char ch) throws IOException
	{
	}


	/**
	 * Writes a char array.
	 * @param buffer a char array
	 * @throws IOException
	 */
	private void write(char[] buffer) throws IOException
	{
	}


	/**
	 * Writes XML characters to the writer.
	 * @param buffer a character array
	 * @param off the offset
	 * @param length the amount of characters to write
	 * @param isAttributeValue flag if it is an attribute value (needed for escaping)
	 * @throws IOException
	 */
	private void writeCharactersInternal(char[] buffer, int off, int length,
			boolean isAttributeValue) throws IOException
	{
	}


	/**
	 * Closes an open tag, either with an end tag or with "/>".
	 * @throws IOException
	 */
	private void closeStartElement() throws IOException
	{
	}


	/**
	 * Writes a closing tag.
	 * @throws IOException
	 */
	private void writeCloseElement() throws IOException
	{
	}


	/**
	 * Writes a newline and indent for the next line.
	 * @throws IOException
	 */
	private void writeNewLine() throws IOException
	{
	}


	/**
	 * Asks if the namespace needs to be written or if it has already been defined.
	 * @param prefix a prefix
	 * @return Returns true if the namespace needs to be written.
	 */
	private boolean needToWriteNamespace(String prefix)
	{
		return false;
	}
}