// =================================================================================================
// ADOBE SYSTEMS INCORPORATED
// Copyright 2012 Adobe Systems Incorporated
// All Rights Reserved
//
// NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the terms
// of the Adobe license agreement accompanying it.
// =================================================================================================
package com.adobe.xmp.path;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This class provides a parser that parses a path string (using prefixes) and a 
 * prefix-to-namespace mapping to construct a {@link XMPPath} object.
 */
public class XMPPathParser
{
	// Pattern to recognize an array index
	private static Pattern ARRAY_INDEX_EXPR = Pattern.compile("\\[(\\d*)\\]");
	// Pattern to recognize a QName
	// TODO This matches more Prefix/Local names than allowed by XML 1.1, should be improved
	private static Pattern PROP_EXPR = Pattern.compile("([^@:/\\[\\]]+):([^:/\\[\\]]+)");
	// Pattern to recognize a simple qualifier property
	private static Pattern QUALIFIER_EXPR = Pattern.compile("@([^:/\\[\\]]+):([^:/\\[\\]]+)");
	// Pattern to recognize a qualifier selector (e.g. specific language)
	private static Pattern QUALIFIER_SELECTOR_EXPR = Pattern.compile("\\[\\?([^:/\\[\\]]+):([^:/\\[\\]]+)=('[^']+'|\"[^\"]+\")\\]");
	
	/**
	 * Creates an {@link XMPPath} object by parsing an XMP path String and creates one path segment 
	 * for each detected segment in the path.
	 * @param path Path string to parse
	 * @param prefixContract A Map that contains the mapping between Namespaces and their prefixes.
	 * This must be provided to interpret the path correctly.
	 * @throws XMPPathParserException If the path has an invalid format or a prefix is found that is not defined in the prefixContract
	 * @throws IllegalArgumentException If path or prefixContract is null.
	 */
	public static XMPPath parse( String path, Map<String, String> prefixContract ) throws XMPPathParserException
	{
		return null;
	}
}
