// =================================================================================================
// ADOBE SYSTEMS INCORPORATED
// Copyright 2012 Adobe Systems Incorporated
// All Rights Reserved
//
// NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the terms
// of the Adobe license agreement accompanying it.
// =================================================================================================
package com.adobe.xmp.path;

/**
 * Represents one segment in a path into the XMP tree.
 * Each segment has to be created through one of the factories and is immutable.
 */
public class XMPPathSegment 
{
	/**
	 * Types of path segments
	 */
	public enum Type
	{
		PROPERTY,			// Any property that consists of namespace and a localname 
		ARRAY_INDEX, 		// An array index which does not have a namespace or localname itself
		QUALIFIER,			// qualifier of a property, also consists of namespace and localname
		QUALIFIER_SELECTOR 	// selects a specific qualifier by its value (e.g. specific language)
	}
	
	private final Type type;
	private final String namespace;
	private final String name;
	private final String value;
	private final int index;

	// Private constructors
	private XMPPathSegment(Type type, String namespace, String name )
	{
		this(type, namespace, name, null, -1 );
	}
	
	private XMPPathSegment(Type type, String namespace, String name, String value )
	{
		this(type, namespace, name, value, -1 );
	}
	
	private XMPPathSegment(Type type, String namespace, String name, int index )
	{
		this(type, namespace, name, null, index );
	}
	
	private XMPPathSegment(Type type, String namespace, String name, String value, int index)
	{
		this.type = type;
		this.namespace = namespace;
		this.name = name;
		this.value = value;
		this.index = index;
	}

	// Factories to create the specific segments
	/**
	 * Creates a normal property path segment. 
	 * These are essentially all properties (simple, struct and arrays).
	 * @param namespace Namespace URI of the property
	 * @param name Local name of the property
	 * @return a new immutable path segment
	 */
	public static XMPPathSegment createPropertySegment( String namespace, String name )
	{
		return null;
	}
	
	/**
	 * Creates an array index path segment that denotes a specific element of an array.
	 * Such segments do not have an own name and inherits the namespace from the Array property itself
	 * @param namespace Namespace URI of the array property
	 * @param index Index of the element
	 * @return a new immutable path segment
	 */
	public static XMPPathSegment createArrayIndexSegment( String namespace, int index )
	{
		return null;
	}
	
	/**
	 * Creates a Qualifier path segment, which behaves like a normal property
	 * @param namespace Namespace URI of the qualifier property
	 * @param name Local name of the qualifier property
	 * @return a new immutable path segment
	 */
	public static XMPPathSegment createQualifierSegment(String namespace, String name)
	{
		return null;
	}
	
	/**
	 * Creates a path segment that selects a specific qualifier by its value.
	 * For example a specific language in a alternative array of languages.
	 * @param namespace Namespace URI of the property
	 * @param name Local name of the property
	 * @param value The value that depicts the specific qualifier
	 * @return a new immutable path segment
	 */
	public static XMPPathSegment createQualifierSelectorSegment(String namespace, String name, String value)
	{
		return null;
	}

	// Getter
	/**
	 * @return the namespace
	 */
	public String getNamespace()
	{
		return null;
	}

	/**
	 * @return the name
	 */
	public String getName()
	{
		return null;
	}

	/**
	 * @return the type
	 */
	public Type getType()
	{
		return null;
	}
	
	public int getIndex()
	{
		return 0;
	}

	/**
	 * @return the value
	 */
	public String getValue()
	{
		return null;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return 0;
	}

	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) 
	{
		return false;
	}
	
}
