/*
 * Copyright 1997-2011 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */

package com.day.cq.activitystreams.api;

import org.apache.sling.api.resource.Resource;

import javax.jcr.Session;
import java.util.Collection;

/**
 * The <code>ActivityService</code> is the main entry point for using the activity streams.
 * <p>
 * It provides functionality for creating, read and writting getting a
 * <code>{@link Activity}</code>, <code>{@link ActivityObject}</code> and <code>{@link ActivityTarget}</code>
 * @deprecated Use {@link com.adobe.granite.activitystreams.ActivityManager} instead.
 */
public interface ActivityService {

    /**
     * Returns an empty activity instance.
     * @return The activity instance
     * @throws ActivityException If an exception occurs during creation
     */
    Activity makeActivity() throws ActivityException {
		return null;
	}

    /**
     * Returns an activity instance parsed from a JSON string.
     * @param json The JSON string representing the activity
     * @return The activity instance
     * @throws ActivityException If an exception occurs during parsing and creation
     */
    Activity makeActivity(String json) throws ActivityException {
		return null;
	}

    /**
     * Returns an empty activity object instance.
     * @return The activity object instance.
     * @throws ActivityException If an exception occurs during creation
     */
    ActivityObject makeActivityObject() throws ActivityException {
		return null;
	}

    /**
     * Returns an activity object instance parsed from a JSON string.
     * @param json The JSON string representing the activity object
     * @return The activity object instance
     * @throws ActivityException If an exception occurs during parsing and creation
     */
    ActivityObject makeActivityObject(String json) throws ActivityException {
		return null;
	}

    /**
     * Returns an empty activity target instance.
     * @return The activity target instance.
     * @throws ActivityException If an exception occurs during creation
     */
    ActivityTarget makeActivityTarget() throws ActivityException {
		return null;
	}

    /**
     * Returns an activity target instance parsed from a JSON string.
     * @param json The JSON string representing the activity target
     * @return The activity target instance
     * @throws ActivityException If an exception occurs during parsing and creation
     */
    ActivityTarget makeActivityTarget(String json) throws ActivityException {
		return null;
	}

    /**
	 * Reads an activity.
	 *
	 * @param resource
	 *            The resource representing the activity
     *
	 * @throws ActivityException
	 *             If an exception occurs while reading the activity.
     * @return The activity object if resource is an activity. Null otherwise.
	 */
    Activity readActivity(Resource resource) throws ActivityException {
		return null;
	}

    /**
	 * Reads all user activities.
	 *
	 * @param userHome
	 *            The user to read the activities from. Might point either to
	 *            the user home or the user's profile. Must not be
	 *            <code>null</code>.
	 * @param stream
	 *            The stream to read from. Can be <code>null</code> to use the
	 *            {@link com.day.cq.activitystreams.api.NameConstants#DEFAULT_HIDDEN_STREAM
	 *            default}.
	 * @return List of {@link com.day.cq.activitystreams.api.Activity Activities}
	 * @throws ActivityException
	 *             If an exception occurs while reading the activities.
	 * @throws IllegalArgumentException
	 *             if the <code>userHome</code> is <code>null</code>.
	 *
	 * @see com.day.cq.activitystreams.api.ActivityService#readActivities(org.apache.sling.api.resource.Resource,
	 *      java.lang.String, java.lang.String)
	 * @see com.day.cq.commons.predicate.PredicateProvider PredicateProvider
	 */
    Collection<Activity> readActivities(Resource userHome, String stream) throws ActivityException {
		return null;
	}

    /**
	 * Reads all user activities.
	 *
	 * @param userHome
	 *            The user to read the activities from. Might point either to
	 *            the user home or the user's profile. Must not be
	 *            <code>null</code>.
	 * @param stream
	 *            The stream to read from. Can be <code>null</code> to use the
	 *            {@link com.day.cq.activitystreams.api.NameConstants#DEFAULT_HIDDEN_STREAM
	 *            default}.
     * @param limit The max number of activities to read
	 * @return List of {@link com.day.cq.activitystreams.api.Activity Activities}
	 * @throws ActivityException
	 *             If an exception occurs while reading the activities.
	 * @throws IllegalArgumentException
	 *             if the <code>userHome</code> is <code>null</code>.
	 *
	 * @see com.day.cq.activitystreams.api.ActivityService#readActivities(org.apache.sling.api.resource.Resource,
	 *      java.lang.String, java.lang.String)
	 * @see com.day.cq.commons.predicate.PredicateProvider PredicateProvider
	 */
    Collection<Activity> readActivities(Resource userHome, String stream, long limit) throws ActivityException {
		return null;
	}

    /**
	 * Reads user activities.
	 *
	 * @param userHome
	 *            The user to read the activities from. Might point either to
	 *            the user home or the user's profile. Must not be
	 *            <code>null</code>.
	 * @param stream
	 *            The stream to read from. Can be <code>null</code> to use the
	 *            {@link com.day.cq.activitystreams.api.NameConstants#DEFAULT_HIDDEN_STREAM
	 *            default}.
	 * @param predicateId
	 *            The predicate id to filter the activities. Uses the
	 *            {@link com.day.cq.commons.predicate.PredicateProvider
	 *            PredicateProvider} to resolve the predicate to use. Can be
	 *            <code>null</code> to omit the use of a predicate.
	 * @return List of {@link com.day.cq.activitystreams.api.Activity Activities}
	 * @throws ActivityException
	 *             If an exception occurs while reading the activities.
	 * @throws IllegalArgumentException
	 *             if the <code>userHome</code> is <code>null</code>.
	 *
	 * @see com.day.cq.activitystreams.api.ActivityService#readActivities(org.apache.sling.api.resource.Resource,
	 *      java.lang.String, java.lang.String)
	 * @see com.day.cq.commons.predicate.PredicateProvider PredicateProvider
	 */
    Collection<Activity> readActivities(Resource userHome, String stream, String predicateId)
            throws ActivityException {
		return null;
	}

    /**
	 * Reads user activities.
*
	 * @param userHome
	 *            The user to read the activities from. Might point either to
	 *            the user home or the user's profile. Must not be
	 *            <code>null</code>.
	 * @param stream
	 *            The stream to read from. Can be <code>null</code> to use the
	 *            {@link com.day.cq.activitystreams.api.NameConstants#DEFAULT_HIDDEN_STREAM
	 *            default}.
	 * @param predicateId
	 *            The predicate id to filter the activities. Uses the
	 *            {@link com.day.cq.commons.predicate.PredicateProvider
	 *            PredicateProvider} to resolve the predicate to use. Can be
	 *            <code>null</code> to omit the use of a predicate.
     * @param limit The max number of activities to read
	 * @return List of {@link com.day.cq.activitystreams.api.Activity Activities}
	 * @throws ActivityException
	 *             If an exception occurs while reading the activities.
	 * @throws IllegalArgumentException
	 *             if the <code>userHome</code> is <code>null</code>.
	 *
	 * @see com.day.cq.activitystreams.api.ActivityService#readActivities(org.apache.sling.api.resource.Resource,
	 *      java.lang.String, java.lang.String)
	 * @see com.day.cq.commons.predicate.PredicateProvider PredicateProvider
	 */
    Collection<Activity> readActivities(Resource userHome, String stream, String predicateId, long limit)
            throws ActivityException {
		return null;
	}

    /**
	 * Writes an activity.
	 *
	 * @param activity
	 *            The activity to write
	 * @throws ActivityException
	 *             If an exception occurs while writing the activity.
	 */
    void writeActivity(Activity activity) throws ActivityException {
	}

    /**
     * Returns if provided session is allowed to write the provided activity.
     * @param session The session to check
     * @param activity The activity to write
     * @return True if session can write the activity, false otherwise.
     */
    public boolean isAllowed(Session session, Activity activity) {
		return false;
	}

}
