/*
 * Copyright 1997-2011 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.dam.handler.standard.msoffice;

import org.apache.poi.hpsf.DocumentSummaryInformation;
import org.apache.poi.hpsf.SummaryInformation;
import org.apache.poi.hpsf.NoPropertySetStreamException;
import org.apache.poi.hpsf.PropertySet;
import org.apache.poi.hpsf.PropertySetFactory;
import org.apache.poi.hpsf.Property;
import org.apache.poi.hpsf.CustomProperty;
import org.apache.poi.poifs.eventfilesystem.POIFSReaderEvent;
import org.apache.poi.poifs.eventfilesystem.POIFSReaderListener;
import org.slf4j.LoggerFactory;

/**
 * Apache POI (POIFS) reader listener for MSOffice 2004 documents
 */
public class MetaDataReaderListener implements POIFSReaderListener {

    private static final org.slf4j.Logger log = LoggerFactory.getLogger(MSOfficeHandler.class);

    /**
     * table of XML name start chars (<= 0xFF)
     */
    private boolean[] xmlNameStartChars;

    /**
     * table of XML name chars (<= 0xFF)
     */
    private boolean[] xmlNameChars;

    private MSOfficeMetadata metadata;

    private boolean debug;

    public MetaDataReaderListener() {
        if (log.isDebugEnabled()) {
            debug = true;
        }
        initCharTables();       // initializing character tables for checking valid XML Name
    }

    public void processPOIFSReaderEvent(POIFSReaderEvent event) {
	}

    public void setMetadata(MSOfficeMetadata metadata) {
	}

    /**
     * Simple check for valid XMLNames. Within ASCII range<br>
     * ":" | [A-Z] | "_" | [a-z] | [#xC0-#xD6] | [#xD8-#xF6]<br>
     * are accepted, above all characters (which is not entirely
     * correct according to the XML Spec.
     *
     * @param name an XML Name
     * @return Return <code>true</code> if the name is correct.
     */
    private boolean isXMLName(String name) {
		return false;
	}

    /**
     * Simple check if a character is a valid XML start name char.
     * All characters according to the XML Spec 1.1 are accepted:
     * http://www.w3.org/TR/xml11/#NT-NameStartChar
     *
     * @param ch a character
     * @return Returns true if the character is a valid first char of an XML name.
     */
    private boolean isNameStartChar(char ch) {
		return false;
	}

    /**
     * Simple check if a character is a valid XML name char
     * (every char except the first one), according to the XML Spec 1.1:
     * http://www.w3.org/TR/xml11/#NT-NameChar
     *
     * @param ch a character
     * @return Returns true if the character is a valid char of an XML name.
     */
    private boolean isNameChar(char ch) {
		return false;
	}

    /**
     * Initializes the char tables for the chars 0x00-0xFF for later use,
     * according to the XML 1.1 specification
     * http://www.w3.org/TR/xml11
     */
    private void initCharTables() {
	}

}
