/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.cq.dam.scene7.api;

import java.util.Map;

import javax.swing.event.ChangeListener;

/**
 * The <code>Scene7Publisher</code> is used to add and remove <code>PublishingJob</code> instances to the Scheduler. A job is executed in a
 * periodic manner within a default interval of <code>JOB_PERIOD</code>, which is 20 seconds by default.
 * @deprecated Explicit publishes are no longer required, please remove publishing from any upload processing.
 */
public interface Scene7Publisher {

    /**
     * Adds a {@link com.day.cq.dam.scene7.impl.PublishingJob} to the {@link org.apache.sling.commons.scheduler.Scheduler}.
     * 
     * @param s7Config
     *            S7 cloud config settings
     * @param path
     *            Path to the asset
     * @param publishStatusQueryMap
     *            Parameter map for to query for assets effected by the Publish
     */
    void addJob(S7Config s7Config, String path, Map<String, String> publishStatusQueryMap) {
	}

    /**
     * Removes a {@link com.day.cq.dam.scene7.impl.PublishingJob} from the {@link org.apache.sling.commons.scheduler.Scheduler}.
     * 
     * @param name
     *            Name of the {@link com.day.cq.dam.scene7.impl.PublishingJob}.
     */
    void removeJob(String name) {
	}

    /**
     * @param publishJobType
     * @param s7Config
     * @return <code>true</code> if publish job of this type is active <code>false</code> otherwise
     */
    boolean isPublishJobActive(String publishJobType, S7Config s7Config) {
		return false;
	}

    /**
     * @param path
     *            - local path from which a jobname is created (null is ok)
     * @param s7Config
     * @return jobHandle or null
     */
    String startPublishJob(String path, S7Config s7Config) {
		return null;
	}

    /**
     * Assumes that the job has completed, checks the disposition of the job
     * 
     * @param originalName
     *            - The originalName of the job (not the jobHandle)
     * @param s7Config
     * @return <code>true</code> if publish job completed successfully, <code>false</code> otherwise
     */
    boolean isPublishJobSuccessByOrignalName(String originalName, S7Config s7Config) {
		return false;
	}

    /**
     * Adds a listener to this service
     * 
     * @param listener
     */
    void addChangeListener(ChangeListener listener) {
	}

    /**
     * Removes a listener to this service
     * 
     * @param listener
     */
    void removeChangeListener(ChangeListener listener) {
	}
}
