/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.search.eval;

import java.util.Iterator;
import java.util.Map.Entry;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;
import javax.jcr.query.Row;

import org.apache.felix.scr.annotations.Component;
import org.apache.jackrabbit.util.Text;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.day.cq.search.Predicate;
import com.day.cq.search.facets.FacetExtractor;
import com.day.cq.search.facets.extractors.DistinctValuesFacetExtractor;
import com.day.text.GlobPattern;

/**
 * <code>JcrPropertyPredicateEvaluator</code> is a generic evaluator for
 * matching a jcr property against a value.
 *
 * <h3>Name:</h3>
 * property
 *
 * <h3>Properties:</h3>
 * <dl>
 * <dt>property</dt>
 * <dd>relative path to property</dd>
 * <dt>depth</dt>
 * <dd>number of additional levels under the parent at which to apply the relative path (for instance, property=size depth=2 will check
 * parent/size, parent/*&#047;size and parent/*&#047;*&#047;size)</dd>
 * <dt>value</dt>
 * <dd>value to check property for</dd>
 * <dt>N_value</dt>
 * <dd>use 1_value, 2_value, ... to check for multiple values (combined with OR by default, with AND if and=true) (since 5.3)</dd>
 * <dt>and</dt>
 * <dd>set to true for combining multiple values (N_value) with AND (since 5.3)</dd>
 * <dt>operation</dt>
 * <dd>"equals" for exact match (default), "unequals" for unequality comparison, "like" for using the jcr:like xpath function (optional),
 * "not" for no match (eg. "not(@prop)" in xpath, value param will be ignored) or "exists" for
 * existence match (value can be true - property must exist, the default - or false - same as "not")</dd>
 * <dl>
 *
 * @since 5.2
 */
@Component(metatype = false, factory="com.day.cq.search.eval.PredicateEvaluator/property")
public class JcrPropertyPredicateEvaluator extends AbstractPredicateEvaluator {

    private static final Logger log = LoggerFactory.getLogger(JcrPropertyPredicateEvaluator.class);

    public static final String PROPERTY = "property";
    public static final String VALUE = "value";
    
    public static final String OPERATION = "operation";
    public static final String OP_EQUALS = "equals";
    public static final String OP_UNEQUALS = "unequals";
    public static final String OP_LIKE = "like";
    public static final String OP_NOT = "not";
    public static final String OP_EXISTS = "exists";
    
    public static final String AND = "and";
    public static final String DEPTH = "depth";

    public static final String STEP = "*/";

    public static final int MAX_NUMBER_OF_VALUES = Integer.MAX_VALUE;

    @Override
    public String getXPathExpression(Predicate p, EvaluationContext context) {
		return null;
	}

    protected String getXPathExpression(String property, String value, String operation, int depth) {
		return null;
	}

    protected String getXPathExpression(String property, String value, String operation) {
		return null;
	}

    /**
     * @deprecated since 5.4; use {@link XPath#getEqualsExpression(String, String)} instead
     */
    protected String getEqualsExpression(String property, String value) {
		return null;
	}

    /**
     * Takes care of converting an operation=exists with a value=false to an
     * operation=not.
     */
    private String getOperation(Predicate p, String operation) {
		return null;
	}

    @Override
    public String[] getOrderByProperties(Predicate p, EvaluationContext context) {
		return null;
	}

    @Override
    public FacetExtractor getFacetExtractor(Predicate p, EvaluationContext context) {
		return null;
	}

    @Override
    public boolean includes(Predicate p, Row row, EvaluationContext context) {
		return false;
	}

    protected boolean includes(Node node, String path, String property, String value, String operation, int depth) {
		return false;
	}

    protected boolean includes(Node node, String path, String property, String value, String operation) {
		return false;
	}
    
    private boolean matches(String value, String operation, String propValue) {
		return false;
	}

    private String convertWildcardsForGlobPattern(String term) {
		return null;
	}

    @Override
    public boolean canXpath(Predicate predicate, EvaluationContext context) {
		return false;
	}

    @Override
    public boolean canFilter(Predicate predicate, EvaluationContext context) {
		return false;
	}
}