/*
 * Copyright 1997-2008 Day Management AG
 * Barfuesserplatz 6, 4001 Basel, Switzerland
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Day Management AG, ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Day.
 */
package com.day.cq.search.facets.extractors;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFactory;

import com.day.cq.search.facets.FacetExtractor;

/**
 * <code>PropertyFacetExtractor</code> is a base class for facet extractors that
 * work on a certain property (incl. relative paths to properties in sub-nodes),
 * specified by the parameter <code>propertyRelPath</code> in the constructor.
 * In addition, subclasses can implement {@link #filter(List, ValueFactory)} for
 * further filtering of the list of values.
 * 
 * @since 5.2
 */
public abstract class PropertyFacetExtractor implements FacetExtractor {

    /**
     * Empty list of properties.
     */
    protected static final List<Property> EMPTY_PROPERTY_LIST = Collections.emptyList();
    
    /**
     * Relative path to the property that provides the value for the facet.
     */
    protected final String propertyRelPath;

    public PropertyFacetExtractor(String propertyRelPath) {
        this.propertyRelPath = propertyRelPath;
    }
    
    /**
     * Called for each value found in a node of the result and that matches the
     * relative property path. This can be multiple values per node as it could
     * be a multi-value property or if multiple properties match the relative
     * property path pattern. 
     * 
     * @param value a value to check in which bucket it fits
     * @throws RepositoryException
     */
    protected abstract void handleValue(Value value) throws RepositoryException {
	}

    /**
     * Filters the <code>values</code> by applying the filter of the definition
     * associated with this facet.
     *
     * @param values the values to filter.
     * @param vf the value factory.
     * @return the filtered values.
     * @throws RepositoryException if an error occurs while reading the values.
     */
    protected abstract List<Value> filter(List<Value> values, ValueFactory vf) throws RepositoryException {
		return null;
	}
    
    public void handleNode(Node node) throws RepositoryException {
	}
    
    /**
     * Gets the values of this facet for the provided <code>node</code>.
     *
     * @param node the node.
     * @return the values of this facet.
     * @throws RepositoryException if an error occurs while reading from the node.
     */
    protected List<Value> getValues(Node node) throws RepositoryException {
		return null;
	}

    /**
     * Returns the properties below <code>node</code> that match the
     * <code>pathPattern</code>.
     *
     * @param node the start node.
     * @param pathPattern a relative path pattern. See
     *          {@link Node#getProperties(String)} for a definition of a pattern.
     *
     * @return the properties that match the path pattern.
     * @throws RepositoryException if an error occurs while reading from the
     *          repository.
     */
    private static Iterator<Property> getProperties(Node node, String pathPattern)
            throws RepositoryException {
				return null;
			}

}
