/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.cq.wcm.designimporter.api;

import com.day.cq.wcm.api.Page;
import com.day.cq.wcm.designimporter.CanvasPageBuildOptions;
import com.day.cq.wcm.designimporter.DesignImportException;
import com.day.cq.wcm.designimporter.DesignImporterContext;

import javax.jcr.RepositoryException;
import java.io.InputStream;

/**
 * Defines contract for building a CQ Page from a raw HTML document stream.
 *
 * <p>
 * The onus of translating the HTML document into a CQ page during the design import
 * process is born by implementations conforming to this interface. CanvasBuilders are
 * pluggable and the filepattern OSGi property defines the pattern of the HTML files
 * that the CanvasBuilder is capable of translating. On encountering HTML files while
 * decompressing the package, the design importer framework iterates over the registered
 * CanvasBuilder implementations and tries to match the HTML file name with the implementations'
 * filepattern. On successful match, the CanvasBuilder is brought into action to construct the
 * CQ Page from there.
 * </p>
 *
 * @deprecated Use {@link CanvasBuilder} instead
 */
@Deprecated
public interface CanvasPageBuilder extends CanvasBuilder {

    String EVENT_PAGE_BUILD_COMPLETE = CanvasBuilder.EVENT_COMPLETE;

    /**
     * Builds the CQ Page for the input HTML document stream and the import context.
     *
     * @param htmlStream The InputStream representing the the HTML document that needs
     *                   to be translated to CQ Page
     * @param designImporterContext The context associated with the design package import request
     *
     * @return The CQ Page resulting from the design import process.
     *
     * @throws RepositoryException When an exception occurs while writing to the repository
     * @throws DesignImportException When an import exception occurs
     *
     * @deprecated Use {@link #build(java.io.InputStream, com.day.cq.wcm.designimporter.DesignImporterContext)} instead
     */
    Page buildPage(InputStream htmlStream, DesignImporterContext designImporterContext) throws RepositoryException,
            DesignImportException {
		return null;
	}

    /**
     * Builds the CQ Page for the input HTML document stream and the import context.
     *
     * @param htmlStream The InputStream representing the the HTML document that needs
     *                   to be translated to CQ Page
     * @param designImporterContext The context associated with the design package import request
     * @param pageBuildOptions Set of option flags to control the way the page(and everything around it) is built
     *
     * @return The CQ Page resulting from the design import process.
     *
     * @throws RepositoryException When an exception occurs while writing to the repository
     * @throws DesignImportException When an import exception occurs
     *
     * @deprecated Use {@link #build(java.io.InputStream, com.day.cq.wcm.designimporter.DesignImporterContext, com.day.cq.wcm.designimporter.CanvasBuildOptions)} instead
     */
    Page buildPage(InputStream htmlStream, DesignImporterContext designImporterContext, CanvasPageBuildOptions pageBuildOptions) throws RepositoryException,
            DesignImportException {
		return null;
	}

}
