/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.commons.query.sql2;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFactory;
import javax.jcr.query.InvalidQueryException;
import javax.jcr.query.qom.BindVariableValue;
import javax.jcr.query.qom.Column;
import javax.jcr.query.qom.Constraint;
import javax.jcr.query.qom.DynamicOperand;
import javax.jcr.query.qom.JoinCondition;
import javax.jcr.query.qom.Literal;
import javax.jcr.query.qom.Ordering;
import javax.jcr.query.qom.PropertyExistence;
import javax.jcr.query.qom.PropertyValue;
import javax.jcr.query.qom.QueryObjectModel;
import javax.jcr.query.qom.QueryObjectModelFactory;
import javax.jcr.query.qom.Selector;
import javax.jcr.query.qom.Source;
import javax.jcr.query.qom.StaticOperand;

import org.apache.jackrabbit.commons.query.qom.JoinType;
import org.apache.jackrabbit.commons.query.qom.Operator;

/**
 * The SQL2 parser can convert a JCR-SQL2 query to a QueryObjectModel.
 */
public class Parser {

    // Character types, used during the tokenizer phase
    private static final int CHAR_END = -1, CHAR_VALUE = 2, CHAR_QUOTED = 3;
    private static final int CHAR_NAME = 4, CHAR_SPECIAL_1 = 5, CHAR_SPECIAL_2 = 6;
    private static final int CHAR_STRING = 7, CHAR_DECIMAL = 8;

    // Token types
    private static final int KEYWORD = 1, IDENTIFIER = 2, PARAMETER = 3, END = 4, VALUE = 5;
    private static final int MINUS = 12, PLUS = 13, OPEN = 14, CLOSE = 15;

    // The query as an array of characters and character types
    private String statement;
    private char[] statementChars;
    private int[] characterTypes;

    // The current state of the parser
    private int parseIndex;
    private int currentTokenType;
    private String currentToken;
    private boolean currentTokenQuoted;
    private Value currentValue;
    private ArrayList<String> expected;

    // The bind variables
    private HashMap<String, BindVariableValue> bindVariables;

    // The list of selectors of this query
    private ArrayList<Selector> selectors;

    // SQL injection protection: if disabled, literals are not allowed
    private boolean allowTextLiterals = true, allowNumberLiterals = true;

    private QueryObjectModelFactory factory;
    private ValueFactory valueFactory;

    /**
     * Create a new parser. A parser can be re-used, but it is not thread safe.
     *
     * @param factory the query object model factory
     * @param valueFactory the value factory
     */
    public Parser(QueryObjectModelFactory factory, ValueFactory valueFactory) {
        this.factory = factory;
        this.valueFactory = valueFactory;
    }

    /**
     * Parse a JCR-SQL2 query and return the query object model
     *
     * @param query the query string
     * @return the query object model
     * @throws RepositoryException if parsing failed
     */
    public QueryObjectModel createQueryObjectModel(String query) throws RepositoryException {
		return null;
	}

    private Selector parseSelector() throws RepositoryException {
		return null;
	}

    private String readName() throws RepositoryException {
		return null;
	}

    private Source parseSource() throws RepositoryException {
		return null;
	}

    private JoinCondition parseJoinCondition() throws RepositoryException {
		return null;
	}

    private Constraint parseConstraint() throws RepositoryException {
		return null;
	}

    private Constraint parseAnd() throws RepositoryException {
		return null;
	}

    private Constraint parseCondition() throws RepositoryException {
		return null;
	}

    private Constraint parseCondition(DynamicOperand left) throws RepositoryException {
		return null;
	}

    private PropertyExistence getPropertyExistence(PropertyValue p) throws InvalidQueryException, RepositoryException {
		return null;
	}

    private Constraint parseConditionFuntionIf(String functionName) throws RepositoryException {
		return null;
	}

    private String readPath() throws RepositoryException {
		return null;
	}

    private DynamicOperand parseDynamicOperand() throws RepositoryException {
		return null;
	}

    private DynamicOperand parseExpressionFunction(String functionName) throws RepositoryException {
		return null;
	}

    private PropertyValue parsePropertyValue(String name) throws RepositoryException {
		return null;
	}

    private StaticOperand parseStaticOperand() throws RepositoryException {
		return null;
	}

    /**
     * Create a literal from a parsed value.
     *
     * @param value the original value
     * @return the literal
     */
    private Literal getUncastLiteral(Value value) throws RepositoryException {
		return null;
	}

    private Value parseCastAs(Value value) throws RepositoryException {
		return null;
	}

    private Ordering[] parseOrder() throws RepositoryException {
		return null;
	}

    private ArrayList<ColumnOrWildcard> parseColumns() throws RepositoryException {
		return null;
	}

    private Column[] resolveColumns(int columnParseIndex, ArrayList<ColumnOrWildcard> list) throws RepositoryException {
		return null;
	}

    private boolean readIf(String token) throws RepositoryException {
		return false;
	}

    private boolean isToken(String token) {
		return false;
	}

    private void read(String expected) throws RepositoryException {
	}

    private String readAny() throws RepositoryException {
		return null;
	}

    private Value readString() throws RepositoryException {
		return null;
	}

    private void addExpected(String token) {
	}

    private void initialize(String query) throws InvalidQueryException {
	}

    private void checkRunOver(int i, int len, int startLoop) throws InvalidQueryException {
	}

    private void read() throws RepositoryException {
	}

    private void readString(int i, char end) throws RepositoryException {
	}

    private void checkLiterals(boolean text) throws InvalidQueryException {
	}

    private void readDecimal(int start, int i) throws RepositoryException {
	}

    private InvalidQueryException getSyntaxError() {
		return null;
	}

    private InvalidQueryException getSyntaxError(String expected) {
		return null;
	}

    /**
     * Represents a column or a wildcard in a SQL expression.
     * This class is temporarily used during parsing.
     */
    static class ColumnOrWildcard {
        String selectorName;
        String propertyName;
        String columnName;
    }

    /**
     * Get the selector name if only one selector exists in the query.
     * If more than one selector exists, an exception is thrown.
     *
     * @param name the property name
     * @return the selector name
     */
    private String getOnlySelectorName(String propertyName) throws RepositoryException {
		return null;
	}

}
