/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.api.jmx;

import org.apache.jackrabbit.oak.api.AuthInfo;

/**
 * MBean providing basic {@code Session} information and statistics.
 */
public interface SessionMBean {
    String TYPE = "SessionStatistics";

    /**
     * @return stack trace from where the session was acquired.
     */
    String getInitStackTrace() {
		return null;
	}

    /**
     * @return {@code AuthInfo} for the user associated with the session.
     */
    AuthInfo getAuthInfo() {
		return null;
	}

    /**
     * @return time stamp from when the session was acquired.
     */
    String getLoginTimeStamp() {
		return null;
	}

    /**
     * @return time stamp from the last read access
     */
    String getLastReadAccess() {
		return null;
	}

    /**
     * @return number of read accesses
     */
    long getReadCount() {
		return 0L;
	}

    /**
     * @return read operations per time
     */
    double getReadRate() {
		return 0.0d;
	}

    /**
     * @return time stamp from the last write access
     */
    String getLastWriteAccess() {
		return null;
	}

    /**
     * @return number of write accesses
     */
    long getWriteCount() {
		return 0L;
	}

    /**
     * @return write operations per time
     */
    double getWriteRate() {
		return 0.0d;
	}

    /**
     * @return time stamp from the last refresh
     */
    String getLastRefresh() {
		return null;
	}

    /**
     * @return description of the refresh strategy
     */
    String getRefreshStrategy() {
		return null;
	}

    /**
     * @return {@code true} iff the session will be refreshed on next access.
     */
    boolean getRefreshPending() {
		return false;
	}

    /**
     * @return number of refresh operations
     */
    long getRefreshCount() {
		return 0L;
	}

    /**
     * @return refresh operations per time
     */
    double getRefreshRate() {
		return 0.0d;
	}

    /**
     * @return time stamp from the last save
     */
    String getLastSave() {
		return null;
	}

    /**
     * @return number of save operations
     */
    long getSaveCount() {
		return 0L;
	}

    /**
     * @return save operations per time
     */
    double getSaveRate() {
		return 0.0d;
	}

    /**
     * @return attributes associated with the session
     */
    String[] getSessionAttributes() {
		return null;
	}

    /**
     * @return  stack trace of the last exception that occurred during a save operation
     */
    String getLastFailedSave() {
		return null;
	}
}
