/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.management;

import static com.google.common.base.Preconditions.checkNotNull;
import static org.apache.jackrabbit.oak.management.ManagementOperation.Status;
import static org.apache.jackrabbit.oak.management.ManagementOperation.Status.failed;
import static org.apache.jackrabbit.oak.management.ManagementOperation.Status.fromCompositeData;
import static org.apache.jackrabbit.oak.management.ManagementOperation.Status.succeeded;
import static org.apache.jackrabbit.oak.management.ManagementOperation.Status.unavailable;

import java.util.List;

import javax.annotation.Nonnull;
import javax.management.openmbean.CompositeData;

import com.google.common.base.Function;
import org.apache.jackrabbit.oak.api.jmx.RepositoryManagementMBean;
import org.apache.jackrabbit.oak.commons.jmx.AnnotatedStandardMBean;
import org.apache.jackrabbit.oak.plugins.backup.FileStoreBackupRestoreMBean;
import org.apache.jackrabbit.oak.plugins.blob.BlobGCMBean;
import org.apache.jackrabbit.oak.plugins.index.property.jmx.PropertyIndexAsyncReindexMBean;
import org.apache.jackrabbit.oak.spi.state.RevisionGCMBean;
import org.apache.jackrabbit.oak.spi.whiteboard.Tracker;
import org.apache.jackrabbit.oak.spi.whiteboard.Whiteboard;

/**
 * Default implementation of the {@link RepositoryManagementMBean} based
 * on a {@link Whiteboard} instance, which is used to look up individual
 * service providers for backup ({@link FileStoreBackupRestoreMBean}), data store
 * garbage collections ({@link BlobGCMBean}) and revision store garbage
 * collections ({@link RevisionGCMBean}).
 */
public class RepositoryManager extends AnnotatedStandardMBean implements RepositoryManagementMBean {
    private final Whiteboard whiteboard;

    public RepositoryManager(@Nonnull Whiteboard whiteboard) {
        super(RepositoryManagementMBean.class);
        this.whiteboard = checkNotNull(whiteboard);
    }

    public String getName() {
		return null;
	}

    private <T> Status execute(Class<T> serviceType, Function<T, Status> operation) {
		return null;
	}

    @Override
    public CompositeData startBackup() {
		return null;
	}

    @Override
    public CompositeData getBackupStatus() {
		return null;
	}

    @Override
    public CompositeData startRestore() {
		return null;
	}

    @Override
    public CompositeData getRestoreStatus() {
		return null;
	}

    @Override
    public CompositeData startDataStoreGC(final boolean markOnly) {
		return null;
	}

    @Override
    public CompositeData getDataStoreGCStatus() {
		return null;
	}

    @Override
    public CompositeData startRevisionGC() {
		return null;
	}

    @Override
    public CompositeData getRevisionGCStatus() {
		return null;
	}

    @Override
    public String checkpoint(final long lifetime) {
		return null;
	}

    @Override
    public CompositeData startPropertyIndexAsyncReindex() {
		return null;
	}

    @Override
    public CompositeData getPropertyIndexAsyncReindexStatus() {
		return null;
	}

}
