/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.plugins.index.lucene;

import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import com.google.common.io.ByteStreams;
import com.google.common.primitives.Ints;
import org.apache.jackrabbit.oak.api.Blob;
import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.spi.state.NodeBuilder;
import org.apache.jackrabbit.oak.util.PerfLogger;
import org.apache.lucene.store.AlreadyClosedException;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.IOContext;
import org.apache.lucene.store.IndexInput;
import org.apache.lucene.store.IndexOutput;
import org.apache.lucene.store.Lock;
import org.apache.lucene.store.LockFactory;
import org.apache.lucene.store.NoLockFactory;
import org.apache.lucene.util.WeakIdentityMap;
import org.slf4j.LoggerFactory;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkElementIndex;
import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.base.Preconditions.checkPositionIndexes;
import static com.google.common.base.Preconditions.checkState;
import static com.google.common.collect.Lists.newArrayList;
import static org.apache.jackrabbit.JcrConstants.JCR_DATA;
import static org.apache.jackrabbit.JcrConstants.JCR_LASTMODIFIED;
import static org.apache.jackrabbit.oak.api.Type.BINARIES;
import static org.apache.jackrabbit.oak.api.Type.STRINGS;
import static org.apache.jackrabbit.oak.plugins.memory.PropertyStates.createProperty;

/**
 * Implementation of the Lucene {@link Directory} (a flat list of files)
 * based on an Oak {@link NodeBuilder}.
 */
class OakDirectory extends Directory {
    static final PerfLogger PERF_LOGGER = new PerfLogger(LoggerFactory.getLogger(OakDirectory.class.getName() + ".perf"));
    static final String PROP_DIR_LISTING = "dirListing";
    static final String PROP_BLOB_SIZE = "blobSize";
    protected final NodeBuilder directoryBuilder;
    private final IndexDefinition definition;
    private LockFactory lockFactory;
    private final boolean readOnly;
    private final Set<String> fileNames = Sets.newConcurrentHashSet();

    public OakDirectory(NodeBuilder directoryBuilder, IndexDefinition definition, boolean readOnly) {
        this.lockFactory = NoLockFactory.getNoLockFactory();
        this.directoryBuilder = directoryBuilder;
        this.definition = definition;
        this.readOnly = readOnly;
        this.fileNames.addAll(getListing());
    }

    @Override
    public String[] listAll() throws IOException {
		return null;
	}

    @Override
    public boolean fileExists(String name) throws IOException {
		return false;
	}

    @Override
    public void deleteFile(String name) throws IOException {
	}

    @Override
    public long fileLength(String name) throws IOException {
		return 0L;
	}

    @Override
    public IndexOutput createOutput(String name, IOContext context)
            throws IOException {
				return null;
			}


    @Override
    public IndexInput openInput(String name, IOContext context)
            throws IOException {
				return null;
			}

    @Override
    public Lock makeLock(String name) {
		return null;
	}

    @Override
    public void clearLock(String name) throws IOException {
	}

    @Override
    public void sync(Collection<String> names) throws IOException {
	}

    @Override
    public void close() throws IOException {
	}

    @Override
    public void setLockFactory(LockFactory lockFactory) throws IOException {
	}

    @Override
    public LockFactory getLockFactory() {
		return null;
	}

    @Override
    public String toString() {
		return null;
	}

    private Set<String> getListing(){
		return null;
	}

    /**
     * Size of the blob entries to which the Lucene files are split.
     * Set to higher than the 4kB inline limit for the BlobStore,
     */
    static final int DEFAULT_BLOB_SIZE = 32 * 1024;

    private static class OakIndexFile {

        private final String name;

        private final NodeBuilder file;

        private final int blobSize;

        private long position = 0;

        private long length;

        private List<Blob> data;

        private boolean dataModified = false;

        private int index = -1;

        private byte[] blob;

        private boolean blobModified = false;

        public OakIndexFile(String name, NodeBuilder file) {
            this.name = name;
            this.file = file;
            this.blobSize = determineBlobSize(file);
            this.blob = new byte[blobSize];

            PropertyState property = file.getProperty(JCR_DATA);
            if (property != null && property.getType() == BINARIES) {
                this.data = newArrayList(property.getValue(BINARIES));
            } else {
                this.data = newArrayList();
            }

            this.length = (long)data.size() * blobSize;
            if (!data.isEmpty()) {
                Blob last = data.get(data.size() - 1);
                this.length -= blobSize - last.length();
            }
        }

        private OakIndexFile(OakIndexFile that) {
            this.name = that.name;
            this.file = that.file;
            this.blobSize = that.blobSize;
            this.blob = new byte[blobSize];

            this.position = that.position;
            this.length = that.length;
            this.data = newArrayList(that.data);
            this.dataModified = that.dataModified;
        }

        private void loadBlob(int i) throws IOException {
		}

        private void flushBlob() throws IOException {
		}

        public void seek(long pos) throws IOException {
		}

        public void readBytes(byte[] b, int offset, int len)
                throws IOException {
				}

        public void writeBytes(byte[] b, int offset, int len)
                throws IOException {
				}

        private static int determineBlobSize(NodeBuilder file){
			return 0;
		}

        public void flush() throws IOException {
		}

        @Override
        public String toString() {
			return null;
		}

    }

    private static class OakIndexInput extends IndexInput {

        private final OakIndexFile file;
        private boolean isClone = false;
        private final WeakIdentityMap<OakIndexInput, Boolean> clones;

        public OakIndexInput(String name, NodeBuilder file) {
            super(name);
            this.file = new OakIndexFile(name, file);
            clones = WeakIdentityMap.newConcurrentHashMap();
        }

        private OakIndexInput(OakIndexInput that) {
            super(that.toString());
            this.file = new OakIndexFile(that.file);
            clones = null;
        }

        @Override
        public OakIndexInput clone() {
			return null;
		}

        @Override
        public void readBytes(byte[] b, int o, int n) throws IOException {
		}

        @Override
        public byte readByte() throws IOException {
			return 0;
		}

        @Override
        public void seek(long pos) throws IOException {
		}

        @Override
        public long length() {
			return 0L;
		}

        @Override
        public long getFilePointer() {
			return 0L;
		}

        @Override
        public void close() {
		}

        private void checkNotClosed() {
		}

    }

    private final class OakIndexOutput extends IndexOutput {

        private final OakIndexFile file;

        public OakIndexOutput(String name, NodeBuilder file) throws IOException {
            this.file = new OakIndexFile(name, file);
        }

        @Override
        public long length() {
			return 0L;
		}

        @Override
        public long getFilePointer() {
			return 0L;
		}

        @Override
        public void seek(long pos) throws IOException {
		}

        @Override
        public void writeBytes(byte[] b, int offset, int length)
                throws IOException {
				}

        @Override
        public void writeByte(byte b) throws IOException {
		}

        @Override
        public void flush() throws IOException {
		}

        @Override
        public void close() throws IOException {
		}

    }

}
