/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.plugins.memory;

import static java.util.Collections.emptyList;
import static org.apache.jackrabbit.oak.spi.state.AbstractNodeState.checkValidName;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;

import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.spi.state.ChildNodeEntry;
import org.apache.jackrabbit.oak.spi.state.NodeBuilder;
import org.apache.jackrabbit.oak.spi.state.NodeState;
import org.apache.jackrabbit.oak.spi.state.NodeStateDiff;

/**
 * Singleton instances of empty and non-existent node states, i.e. ones
 * with neither properties nor child nodes.
 */
public final class EmptyNodeState implements NodeState {

    public static final NodeState EMPTY_NODE = new EmptyNodeState(true);

    public static final NodeState MISSING_NODE = new EmptyNodeState(false);

    private final boolean exists;

    private EmptyNodeState(boolean exists) {
        this.exists = exists;
    }

    @Override
    public boolean exists() {
		return false;
	}

    @Override
    public long getPropertyCount() {
		return 0L;
	}

    @Override
    public boolean hasProperty(@Nonnull String name) {
		return false;
	}

    @Override @CheckForNull
    public PropertyState getProperty(@Nonnull String name) {
		return null;
	}

    @Override
    public boolean getBoolean(@Nonnull String name) {
		return false;
	}

    @Override
    public long getLong(String name) {
		return 0L;
	}

    @Override
    public String getString(String name) {
		return null;
	}

    @Nonnull
    @Override
    public Iterable<String> getStrings(@Nonnull String name) {
		return null;
	}

    @Override @CheckForNull
    public String getName(@Nonnull String name) {
		return null;
	}

    @Override @Nonnull
    public Iterable<String> getNames(@Nonnull String name) {
		return null;
	}

    @Override @Nonnull
    public Iterable<? extends PropertyState> getProperties() {
		return null;
	}

    @Override
    public long getChildNodeCount(long max) {
		return 0L;
	}

    @Override
    public boolean hasChildNode(@Nonnull String name) {
		return false;
	}

    @Override @Nonnull
    public NodeState getChildNode(@Nonnull String name) {
		return null;
	}

    @Override
    public Iterable<String> getChildNodeNames() {
		return null;
	}

    @Override @Nonnull
    public Iterable<? extends ChildNodeEntry> getChildNodeEntries() {
		return null;
	}

    @Override @Nonnull
    public NodeBuilder builder() {
		return null;
	}

    @Override
    public boolean compareAgainstBaseState(NodeState base, NodeStateDiff diff) {
		return false;
	}

    public static boolean compareAgainstEmptyState(
            NodeState state, NodeStateDiff diff) {
				return false;
			}

    public static boolean isEmptyState(NodeState state) {
		return false;
	}

    //------------------------------------------------------------< Object >--

    public String toString() {
		return null;
	}

    public boolean equals(Object object) {
		return false;
	}

    public int hashCode() {
		return 0;
	}

}
