/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.plugins.memory;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkState;

import java.math.BigDecimal;
import java.util.List;

import javax.annotation.Nonnull;
import javax.jcr.PropertyType;

import com.google.common.collect.Lists;
import org.apache.jackrabbit.oak.api.Blob;
import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.api.Type;

/**
 * {@code PropertyBuilder} for building in memory {@code PropertyState} instances.
 * @param <T>
 */
public class PropertyBuilder<T> {
    private final Type<T> type;

    private String name;
    private boolean isArray;
    private List<T> values = Lists.newArrayList();

    /**
     * Create a new instance for building {@code PropertyState} instances
     * of the given {@code type}.
     * @param type  type of the {@code PropertyState} instances to be built.
     * @throws IllegalArgumentException if {@code type.isArray()} is {@code true}.
     */
    public PropertyBuilder(Type<T> type) {
        checkArgument(!type.isArray(), "type must not be array");
        this.type = type;
    }

    /**
     * Create a new instance for building scalar {@code PropertyState} instances
     * of the given {@code type}.
     * @param type  type of the {@code PropertyState} instances to be built.
     * @return {@code PropertyBuilder} for {@code type}
     */
    public static <T> PropertyBuilder<T> scalar(Type<T> type) {
		return null;
	}

    /**
     * Create a new instance for building array {@code PropertyState} instances
     * of the given {@code type}.
     * @param type  type of the {@code PropertyState} instances to be built.
     * @return {@code PropertyBuilder} for {@code type}
     */
    public static <T> PropertyBuilder<T> array(Type<T> type) {
		return null;
	}

    /**
     * Create a new instance for building scalar {@code PropertyState} instances
     * of the given {@code type}. The builder is initialised with the
     * given {@code name}.
     * Equivalent to
     * <pre>
     *     MemoryPropertyBuilder.create(type).setName(name);
     * </pre>
     * @param type  type of the {@code PropertyState} instances to be built.
     * @param name  initial name
     * @return {@code PropertyBuilder} for {@code type}
     */
    public static <T> PropertyBuilder<T> scalar(Type<T> type, String name) {
		return null;
	}

    /**
     * Create a new instance for building array {@code PropertyState} instances
     * of the given {@code type}. The builder is initialised with the
     * given {@code name}.
     * Equivalent to
     * <pre>
     *     MemoryPropertyBuilder.create(type).setName(name).setArray();
     * </pre>
     * @param type  type of the {@code PropertyState} instances to be built.
     * @param name  initial name
     * @return {@code PropertyBuilder} for {@code type}
     */
    public static <T> PropertyBuilder<T> array(Type<T> type, String name) {
		return null;
	}

    /**
     * Create a new instance for building {@code PropertyState} instances
     * of the given {@code type}. The builder is initialised with the name and
     * the values of {@code property}.
     * Equivalent to
     * <pre>
     *     PropertyBuilder.scalar(type).assignFrom(property);
     * </pre>
     *
     * @param type  type of the {@code PropertyState} instances to be built.
     * @param property  initial name and values
     * @return {@code PropertyBuilder} for {@code type}
     */
    public static <T> PropertyBuilder<T> copy(Type<T> type, PropertyState property) {
		return null;
	}

    public String getName() {
		return null;
	}

    public T getValue() {
		return null;
	}

    @Nonnull
    public List<T> getValues() {
		return null;
	}

    public T getValue(int index) {
		return null;
	}

    public boolean hasValue(Object value) {
		return false;
	}

    public int count() {
		return 0;
	}

    public boolean isArray() {
		return false;
	}

    public boolean isEmpty() {
		return false;
	}

    @SuppressWarnings("unchecked")
    @Nonnull
    public PropertyState getPropertyState() {
		return null;
	}

    @SuppressWarnings("unchecked")
    @Nonnull
    public PropertyBuilder<T> assignFrom(PropertyState property) {
		return null;
	}

    @Nonnull
    public PropertyBuilder<T> setName(String name) {
		return null;
	}

    @Nonnull
    public PropertyBuilder<T> setArray() {
		return null;
	}

    @Nonnull
    public PropertyBuilder<T> setScalar() {
		return null;
	}

    @Nonnull
    public PropertyBuilder<T> setValue(T value) {
		return null;
	}

    @Nonnull
    public PropertyBuilder<T> addValue(T value) {
		return null;
	}

    @Nonnull
    public PropertyBuilder<T> setValue(T value, int index) {
		return null;
	}

    @Nonnull
    public PropertyBuilder<T> setValues(Iterable<T> values) {
		return null;
	}

    @Nonnull
    public PropertyBuilder<T> removeValue(int index) {
		return null;
	}

    @Nonnull
    public PropertyBuilder<T> removeValue(Object value) {
		return null;
	}
}
