/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.jackrabbit.oak.plugins.segment;

import static com.google.common.base.Preconditions.checkArgument;
import static org.apache.commons.io.FileUtils.byteCountToDisplaySize;
import static org.apache.jackrabbit.oak.api.Type.BINARY;
import static org.apache.jackrabbit.oak.plugins.segment.ListRecord.LEVEL_SIZE;
import static org.apache.jackrabbit.oak.plugins.segment.Segment.MEDIUM_LIMIT;
import static org.apache.jackrabbit.oak.plugins.segment.Segment.RECORD_ID_BYTES;
import static org.apache.jackrabbit.oak.plugins.segment.Segment.SMALL_LIMIT;
import static org.apache.jackrabbit.oak.plugins.segment.SegmentVersion.V_11;
import static org.apache.jackrabbit.oak.plugins.segment.SegmentWriter.BLOCK_SIZE;
import static org.apache.jackrabbit.oak.plugins.segment.Template.MANY_CHILD_NODES;
import static org.apache.jackrabbit.oak.plugins.segment.Template.ZERO_CHILD_NODES;

import java.util.Formatter;

import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.spi.state.ChildNodeEntry;
import org.apache.jackrabbit.oak.spi.state.NodeState;

/**
 * This utility breaks down space usage per record type.
 * It accounts for value sharing. That is, an instance
 * of this class will remember which records it has seen
 * already and not count those again. Only the effective
 * space taken by the records is taken into account. Slack
 * space from aligning records is not accounted for.
 */
public class RecordUsageAnalyser {
    private final RecordIdSet seenIds = new RecordIdSet();

    private long mapSize;       // leaf and branch
    private long listSize;      // list and bucket
    private long valueSize;     // inlined values
    private long templateSize;  // template
    private long nodeSize;      // node

    private long mapCount;
    private long listCount;
    private long propertyCount;
    private long smallBlobCount;
    private long mediumBlobCount;
    private long longBlobCount;
    private long externalBlobCount;
    private long smallStringCount;
    private long mediumStringCount;
    private long longStringCount;
    private long templateCount;
    private long nodeCount;

    /**
     * @return number of bytes in {@link RecordType#LEAF leaf} and {@link RecordType#BRANCH branch}
     * records.
     */
    public long getMapSize() {
		return 0L;
	}

    /**
     * @return number of bytes in {@link RecordType#LIST list} and {@link RecordType#BUCKET bucket}
     * records.
     */
    public long getListSize() {
		return 0L;
	}

    /**
     * @return number of bytes in inlined values (strings and blobs)
     */
    public long getValueSize() {
		return 0L;
	}

    /**
     * @return number of bytes in {@link RecordType#TEMPLATE template} records.
     */
    public long getTemplateSize() {
		return 0L;
	}

    /**
     * @return number of bytes in {@link RecordType#NODE node} records.
     */
    public long getNodeSize() {
		return 0L;
	}

    /**
     * @return number of maps
     */
    public long getMapCount() {
		return 0L;
	}

    /**
     * @return number of lists
     */
    public long getListCount() {
		return 0L;
	}

    /**
     * @return number of properties
     */
    public long getPropertyCount() {
		return 0L;
	}

    /**
     * @return number of {@link Segment#SMALL_LIMIT small}blobs.
     *
     */
    public long getSmallBlobCount() {
		return 0L;
	}

    /**
     * @return number of {@link Segment#MEDIUM_LIMIT medium}blobs.
     *
     */
    public long getMediumBlobCount() {
		return 0L;
	}

    /**
     * @return number of longblobs.
     *
     */
    public long getLongBlobCount() {
		return 0L;
	}

    /**
     * @return number of externalblobs.
     *
     */
    public long getExternalBlobCount() {
		return 0L;
	}

    /**
     * @return number of {@link Segment#SMALL_LIMIT small}strings.
     *
     */
    public long getSmallStringCount() {
		return 0L;
	}

    /**
     * @return number of {@link Segment#MEDIUM_LIMIT medium}strings.
     *
     */
    public long getMediumStringCount() {
		return 0L;
	}

    /**
     * @return number of longstrings.
     *
     */
    public long getLongStringCount() {
		return 0L;
	}

    /**
     * @return number of templates.
     */
    public long getTemplateCount() {
		return 0L;
	}

    /**
     * @return number of nodes.
     */
    public long getNodeCount() {
		return 0L;
	}

    public void analyseNode(RecordId nodeId) {
	}

    @Override
    public String toString() {
		return null;
	}

    private void analyseTemplate(RecordId templateId) {
	}

    private void analyseMap(RecordId mapId, MapRecord map) {
	}

    private void analyseDiff(RecordId mapId, MapRecord map) {
	}

    private void analyseLeaf(MapRecord map) {
	}

    private void analyseBranch(MapRecord map) {
	}

    private void analyseProperty(RecordId propertyId, PropertyTemplate template) {
	}

    private void analyseValue(RecordId valueId, Type<?> type) {
	}

    private void analyseBlob(RecordId blobId) {
	}

    private void analyseString(RecordId stringId) {
	}

    private void analyseList(RecordId listId, int size) {
	}

    private static int noOfListSlots(int size) {
		return 0;
	}

}
