/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.plugins.version;

import javax.annotation.Nonnull;

import org.apache.jackrabbit.oak.api.CommitFailedException;
import org.apache.jackrabbit.oak.api.PropertyState;
import org.apache.jackrabbit.oak.spi.commit.DefaultEditor;
import org.apache.jackrabbit.oak.spi.commit.Editor;
import org.apache.jackrabbit.oak.spi.state.NodeBuilder;
import org.apache.jackrabbit.oak.spi.state.NodeState;

import static com.google.common.base.Preconditions.checkNotNull;
import static org.apache.jackrabbit.JcrConstants.JCR_PRIMARYTYPE;
import static org.apache.jackrabbit.JcrConstants.JCR_VERSIONLABELS;
import static org.apache.jackrabbit.oak.commons.PathUtils.concat;
import static org.apache.jackrabbit.oak.commons.PathUtils.getDepth;
import static org.apache.jackrabbit.oak.commons.PathUtils.relativize;
import static org.apache.jackrabbit.oak.plugins.version.VersionConstants.VERSION_NODE_TYPE_NAMES;
import static org.apache.jackrabbit.oak.plugins.version.VersionConstants.VERSION_STORE_NT_NAMES;
import static org.apache.jackrabbit.oak.plugins.version.VersionConstants.VERSION_STORE_PATH;

/**
 * Validates changes on the version store.
 */
class VersionStorageEditor extends DefaultEditor {

    private static final int VERSION_HISTORY_DEPTH = 6;

    private final NodeBuilder versionStorageNode;
    private final NodeBuilder workspaceRoot;
    private final NodeBuilder builder;
    private final String path;
    private ReadWriteVersionManager vMgr;

    VersionStorageEditor(@Nonnull NodeBuilder versionStorageNode,
                         @Nonnull NodeBuilder workspaceRoot) {
        this(versionStorageNode, workspaceRoot, versionStorageNode,
                VERSION_STORE_PATH);
    }

    private VersionStorageEditor(@Nonnull NodeBuilder versionStorageNode,
                                 @Nonnull NodeBuilder workspaceRoot,
                                 @Nonnull NodeBuilder builder,
                                 @Nonnull String path) {
        this.versionStorageNode = checkNotNull(versionStorageNode);
        this.workspaceRoot = checkNotNull(workspaceRoot);
        this.builder = checkNotNull(builder);
        this.path = checkNotNull(path);
    }

    @Override
    public Editor childNodeChanged(String name,
                                   NodeState before,
                                   NodeState after)
            throws CommitFailedException {
				return null;
			}

    @Override
    public Editor childNodeAdded(String name, NodeState after)
            throws CommitFailedException {
				return null;
			}

    @Override
    public Editor childNodeDeleted(String name, NodeState before)
            throws CommitFailedException {
				return null;
			}

    @Override
    public void propertyAdded(PropertyState after)
            throws CommitFailedException {
			}

    @Override
    public void propertyChanged(PropertyState before, PropertyState after)
            throws CommitFailedException {
			}

    @Override
    public void propertyDeleted(PropertyState before)
            throws CommitFailedException {
			}

    //-------------------------< internal >-------------------------------------

    private static boolean isVersionStorageNode(NodeState state) {
		return false;
	}

    private ReadWriteVersionManager getVersionManager() {
		return null;
	}

    private Editor throwProtected(String name) throws CommitFailedException {
		return null;
	}
}
