/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.client;

import java.rmi.RemoteException;

import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeIterator;
import javax.jcr.nodetype.PropertyDefinition;

import org.apache.jackrabbit.rmi.remote.RemoteNodeDefinition;
import org.apache.jackrabbit.rmi.remote.RemoteNodeType;
import org.apache.jackrabbit.rmi.remote.RemotePropertyDefinition;
import org.apache.jackrabbit.rmi.value.SerialValueFactory;

/**
 * Local adapter for the JCR-RMI
 * {@link org.apache.jackrabbit.rmi.remote.RemoteNodeType RemoteNodeType}
 * interface. This class makes a remote node type locally available using
 * the JCR {@link javax.jcr.nodetype.NodeType NodeType} interface.
 *
 * @see javax.jcr.nodetype.NodeType
 * @see org.apache.jackrabbit.rmi.remote.RemoteNodeType
 */
public class ClientNodeType extends ClientObject implements NodeType {

    /** The adapted remote node type. */
    private RemoteNodeType remote;

    /**
     * Creates a local adapter for the given remote node type.
     *
     * @param remote remote node type
     * @param factory local adapter factory
     */
    public ClientNodeType(RemoteNodeType remote, LocalAdapterFactory factory) {
        super(factory);
        this.remote = remote;
    }

    /**
     * Utility method for creating an array of local node definition
     * adapters for an array of remote node definitions. The node
     * definition adapters are created using the local adapter factory.
     * <p>
     * A <code>null</code> input is treated as an empty array.
     *
     * @param remotes remote node definitions
     * @return local node definition array
     */
    private NodeDefinition[] getNodeDefArray(RemoteNodeDefinition[] remotes) {
		return null;
	}

    /**
     * Utility method for creating an array of local property definition
     * adapters for an array of remote property definitions. The property
     * definition adapters are created using the local adapter factory.
     * <p>
     * A <code>null</code> input is treated as an empty array.
     *
     * @param remotes remote property definitions
     * @return local property definition array
     */
    protected PropertyDefinition[] getPropertyDefArray(
            RemotePropertyDefinition[] remotes) {
				return null;
			}

    /** {@inheritDoc} */
    public String getName() {
		return null;
	}

    /** {@inheritDoc} */
    public boolean isMixin() {
		return false;
	}

    /** {@inheritDoc} */
    public boolean hasOrderableChildNodes() {
		return false;
	}

    /** {@inheritDoc} */
    public NodeType[] getSupertypes() {
		return null;
	}

    /** {@inheritDoc} */
    public NodeType[] getDeclaredSupertypes() {
		return null;
	}

    /** {@inheritDoc} */
    public boolean isNodeType(String type) {
		return false;
	}

    /** {@inheritDoc} */
    public PropertyDefinition[] getPropertyDefinitions() {
		return null;
	}

    /** {@inheritDoc} */
    public PropertyDefinition[] getDeclaredPropertyDefinitions() {
		return null;
	}

    /** {@inheritDoc} */
    public NodeDefinition[] getChildNodeDefinitions() {
		return null;
	}

    /** {@inheritDoc} */
    public NodeDefinition[] getDeclaredChildNodeDefinitions() {
		return null;
	}

    /** {@inheritDoc} */
    public boolean canSetProperty(String name, Value value) {
		return false;
	}

    /** {@inheritDoc} */
    public boolean canSetProperty(String name, Value[] values) {
		return false;
	}

    /** {@inheritDoc} */
    public boolean canAddChildNode(String name) {
		return false;
	}

    /** {@inheritDoc} */
    public boolean canAddChildNode(String name, String type) {
		return false;
	}

    /** {@inheritDoc} */
    public boolean canRemoveItem(String name) {
		return false;
	}

    /** {@inheritDoc} */
    public String getPrimaryItemName() {
		return null;
	}

    /** {@inheritDoc} */
    public boolean canRemoveNode(String nodeName) {
		return false;
	}

    /** {@inheritDoc} */
    public boolean canRemoveProperty(String propertyName) {
		return false;
	}

    /** {@inheritDoc} */
    public NodeTypeIterator getDeclaredSubtypes() {
		return null;
	}

    /** {@inheritDoc} */
    public NodeTypeIterator getSubtypes() {
		return null;
	}

    /** {@inheritDoc} */
    public String[] getDeclaredSupertypeNames() {
		return null;
	}

    /** {@inheritDoc} */
    public boolean isAbstract() {
		return false;
	}

    /** {@inheritDoc} */
    public boolean isQueryable() {
		return false;
	}
}
