/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.client;

import java.rmi.RemoteException;
import java.util.Calendar;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.version.Version;
import javax.jcr.version.VersionHistory;

import org.apache.jackrabbit.rmi.remote.RemoteVersion;

/**
 * Local adapter for the JCR-RMI
 * {@link org.apache.jackrabbit.rmi.remote.RemoteVersion RemoteVersion}
 * interface. This class makes a remote version locally available using
 * the JCR {@link javax.jcr.version.Version Version} interface.
 *
 * @see javax.jcr.version.Version
 * @see org.apache.jackrabbit.rmi.remote.RemoteVersion
 */
public class ClientVersion extends ClientNode implements Version {

    /** The adapted remote version. */
    private RemoteVersion remote;

    /**
     * Creates a local adapter for the given remote version.
     *
     * @param session current session
     * @param remote  remote version
     * @param factory local adapter factory
     */
    public ClientVersion(Session session, RemoteVersion remote,
        LocalAdapterFactory factory) {
        super(session, remote, factory);
        this.remote = remote;
    }

    /**
     * Utility method for creating a version array for an array
     * of remote versions. The versions in the returned array
     * are created using the local adapter factory.
     * <p>
     * A <code>null</code> input is treated as an empty array.
     *
     * @param remotes remote versions
     * @return local version array
     */
    private Version[] getVersionArray(RemoteVersion[] remotes) {
		return null;
	}


    /** {@inheritDoc} */
    public Calendar getCreated() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Version[] getSuccessors() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Version[] getPredecessors() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public VersionHistory getContainingHistory() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Node getFrozenNode() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Version getLinearPredecessor() throws RepositoryException {
		return null;
	}

    /** {@inheritDoc} */
    public Version getLinearSuccessor() throws RepositoryException {
		return null;
	}
}
