/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.rmi.value;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.io.Serializable;

import javax.jcr.Binary;
import javax.jcr.RepositoryException;

/**
 * Serializable binary.
 */
class SerializableBinary implements Binary, Serializable {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = -7742179594834275853L;

    private static final int BUFFER_SIZE = 64 * 1024;

    private transient long length;

    private transient byte[] data;

    private transient File file;

    /**
     * Creates a binary from the given stream. The stream is not closed.
     *
     * @param stream binary stream
     */
    public SerializableBinary(InputStream stream) throws IOException {
        length = 0;
        data = null;
        file = null;

        OutputStream output = null;
        try {
            byte[] buffer = new byte[BUFFER_SIZE];
            int n = stream.read(buffer);
            while (n != -1) {
                length += n;
                if (length < buffer.length) {
                    n = stream.read(
                            buffer, (int) length, buffer.length - (int) length);
                } else {
                    if (file == null) {
                        file = File.createTempFile("jackrabbit-jcr-rmi-", null);
                        output = new FileOutputStream(file);
                        output.write(buffer);
                    } else {
                        output.write(buffer, 0, n);
                    }
                    n = stream.read(buffer);
                }
            }
            if (file == null) {
                data = new byte[(int) length];
                System.arraycopy(buffer, 0, data, 0, (int) length);
            }
        } finally {
            if (output != null) {
                output.close();
            }
        }
    }

    public synchronized int read(byte[] b, long position)
            throws RepositoryException {
				return 0;
			}

    public synchronized InputStream getStream() throws RepositoryException {
		return null;
	}

    public long getSize() {
		return 0L;
	}

    public synchronized void dispose() {
	}

    private synchronized void writeObject(ObjectOutputStream stream)
            throws IOException {
			}

    private void readObject(ObjectInputStream stream)
            throws IOException {
			}

    public void finalize() {
	}

}
