/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.conversion;

import org.apache.jackrabbit.spi.NameFactory;
import org.apache.jackrabbit.spi.Name;
import org.apache.jackrabbit.spi.commons.namespace.NamespaceResolver;
import org.apache.jackrabbit.util.XMLChar;

import javax.jcr.NamespaceException;

/**
 * <code>NameParser</code> parses a {@link String jcrName} using a
 * {@link NamespaceResolver} and a {@link NameFactory}.
 */
public class NameParser {

    // constants for parser
    private static final int STATE_PREFIX_START = 0;
    private static final int STATE_PREFIX = 1;
    private static final int STATE_NAME_START = 2;
    private static final int STATE_NAME = 3;
    private static final int STATE_URI_START = 4;
    private static final int STATE_URI = 5;

    /**
     * Parses the <code>jcrName</code> (either qualified or expanded) and
     * returns a new <code>Name</code>.
     *
     * @param jcrName the name to be parsed. The jcrName may either be in the
     * qualified or in the expanded form.
     * @param resolver <code>NamespaceResolver</code> use to retrieve the
     * namespace URI from the prefix contained in the given JCR name.
     * @return qName the new <code>Name</code>
     * @throws IllegalNameException If <code>jcrName</code> is not a valid
     * JCR-style name.
     * @throws NamespaceException If the jcr name contains an unknown prefix.
     */
    public static Name parse(String jcrName, NamespaceResolver resolver, NameFactory factory)
            throws IllegalNameException, NamespaceException {
				return null;
			}

    /**
     * Parses an array of <code>jcrName</code> and returns the respective
     * array of <code>Name</code>.
     *
     * @param jcrNames the array of names to be parsed
     * @param resolver <code>NamespaceResolver</code> use to retrieve the
     * namespace URI from the prefix contained in the given JCR name.
     * @param factory
     * @return the new array of <code>Name</code>
     * @throws IllegalNameException If <code>jcrName</code> is not a valid
     * JCR-style name.
     * @throws NamespaceException If the jcr name contains an unknown prefix.
     */
    public static Name[] parse(String jcrNames[], NamespaceResolver resolver, NameFactory factory)
            throws NameException, NamespaceException {
				return null;
			}

    /**
     * Check the format of the jcr name. Note that the prefix is not resolved
     * and therefore namespace violations (unknown prefix) will not be detected.
     *
     * @param jcrName
     * @throws IllegalNameException If the jcrName contains an invalid format.
     */
    public static void checkFormat(String jcrName) throws IllegalNameException {
	}
}
