/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.nodetype;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.NamespaceException;
import javax.jcr.RepositoryException;

import org.apache.jackrabbit.commons.cnd.DefinitionBuilderFactory;
import org.apache.jackrabbit.commons.cnd.CompactNodeTypeDefReader;
import org.apache.jackrabbit.spi.Name;
import org.apache.jackrabbit.spi.NameFactory;
import org.apache.jackrabbit.spi.QNodeDefinition;
import org.apache.jackrabbit.spi.QNodeTypeDefinition;
import org.apache.jackrabbit.spi.QPropertyDefinition;
import org.apache.jackrabbit.spi.commons.QNodeTypeDefinitionImpl;
import org.apache.jackrabbit.spi.commons.conversion.DefaultNamePathResolver;
import org.apache.jackrabbit.spi.commons.conversion.IllegalNameException;
import org.apache.jackrabbit.spi.commons.conversion.NamePathResolver;
import org.apache.jackrabbit.spi.commons.name.NameConstants;
import org.apache.jackrabbit.spi.commons.name.NameFactoryImpl;
import org.apache.jackrabbit.spi.commons.namespace.NamespaceMapping;
import org.apache.jackrabbit.spi.commons.nodetype.constraint.ValueConstraint;
import org.apache.jackrabbit.spi.commons.value.QValueFactoryImpl;
import org.apache.jackrabbit.spi.commons.value.ValueFormat;
import org.apache.jackrabbit.util.ISO9075;

/**
 * This implementation of {@link DefinitionBuilderFactory} can be used with
 * the {@link CompactNodeTypeDefReader} to produce node type definitions of type
 * {@link QNodeTypeDefinition} and a namespace map of type {@link NamespaceMapping}.
 * It uses {@link QNodeTypeDefinitionBuilderImpl} for building node type definitions,
 * {@link QPropertyDefinitionBuilderImpl} for building property definitions, and
 * {@link QNodeDefinitionBuilderImpl} for building node definitions. It further uses
 * {@link NameFactoryImpl} for creating <code>Name</code>s and {@link QValueFactoryImpl} for
 * creating <code>QValue</code>s.
 */
public class QDefinitionBuilderFactory extends DefinitionBuilderFactory<QNodeTypeDefinition, NamespaceMapping> {

    private static final NameFactory NAME_FACTORY = NameFactoryImpl.getInstance();

    /**
     * Default namespace mappings
     */
    public static final NamespaceMapping NS_DEFAULTS;
    static {
        try {
            NS_DEFAULTS = new NamespaceMapping();
            NS_DEFAULTS.setMapping(Name.NS_EMPTY_PREFIX, Name.NS_DEFAULT_URI);
            NS_DEFAULTS.setMapping(Name.NS_JCR_PREFIX, Name.NS_JCR_URI);
            NS_DEFAULTS.setMapping(Name.NS_MIX_PREFIX, Name.NS_MIX_URI);
            NS_DEFAULTS.setMapping(Name.NS_NT_PREFIX, Name.NS_NT_URI);
            NS_DEFAULTS.setMapping(Name.NS_REP_PREFIX, Name.NS_REP_URI);
        } catch (NamespaceException e) {
            throw new InternalError(e.toString());
        }
    }

    private NamespaceMapping nsMappings = new NamespaceMapping(NS_DEFAULTS);
    private NamePathResolver resolver = new DefaultNamePathResolver(nsMappings);

    @Override
    public AbstractNodeTypeDefinitionBuilder<QNodeTypeDefinition> newNodeTypeDefinitionBuilder() {
		return null;
	}

    @Override
    public void setNamespaceMapping(NamespaceMapping nsMapping) {
	}

    @Override
    public NamespaceMapping getNamespaceMapping() {
		return null;
	}

    @Override
    public void setNamespace(String prefix, String uri) {
	}

    private class QNodeTypeDefinitionBuilderImpl extends AbstractNodeTypeDefinitionBuilder<QNodeTypeDefinition> {
        private Name name;
        private final List<Name> supertypes = new ArrayList<Name>();
        private Name primaryItem;
        private final List<QPropertyDefinition> propertyDefs = new ArrayList<QPropertyDefinition>();
        private final List<QNodeDefinition> childNodeDefs = new ArrayList<QNodeDefinition>();

        @Override
        public AbstractNodeDefinitionBuilder<QNodeTypeDefinition> newNodeDefinitionBuilder() {
			return null;
		}

        @Override
        public AbstractPropertyDefinitionBuilder<QNodeTypeDefinition> newPropertyDefinitionBuilder() {
			return null;
		}

        @Override
        public QNodeTypeDefinition build() {
			return null;
		}

        @Override
        public void setName(String name) throws RepositoryException {
		}

        @Override
        public void addSupertype(String name) throws IllegalNameException, NamespaceException {
		}

        @Override
        public void setPrimaryItemName(String name) throws IllegalNameException, NamespaceException {
		}

    }

    private class QPropertyDefinitionBuilderImpl extends AbstractPropertyDefinitionBuilder<QNodeTypeDefinition> {

        private final QNodeTypeDefinitionBuilderImpl ntd;
        private final QPropertyDefinitionBuilder builder = new QPropertyDefinitionBuilder();

        public QPropertyDefinitionBuilderImpl(QNodeTypeDefinitionBuilderImpl ntd) {
            super();
            this.ntd = ntd;
        }

        @Override
        public void setName(String name) throws RepositoryException {
		}

        @Override
        public void setRequiredType(int type) throws RepositoryException {
		}

        @Override
        public void setMultiple(boolean isMultiple) throws RepositoryException {
		}

        @Override
        public void setFullTextSearchable(boolean fullTextSearchable)
                throws RepositoryException {
				}

        @Override
        public void setQueryOrderable(boolean queryOrderable)
                throws RepositoryException {
				}

        @Override
        public void setAvailableQueryOperators(String[] queryOperators)
                throws RepositoryException {
				}

        @Override
        public void setAutoCreated(boolean autocreate)
                throws RepositoryException {
				}

        @Override
        public void setOnParentVersion(int onParent)
                throws RepositoryException {
				}

        @Override
        public void setProtected(boolean isProtected)
                throws RepositoryException {
				}

        @Override
        public void setMandatory(boolean isMandatory)
                throws RepositoryException {
				}

        @Override
        public void addDefaultValues(String value) throws RepositoryException {
		}

        @Override
        public void addValueConstraint(String constraint) throws InvalidConstraintException {
		}

        @Override
        public void setDeclaringNodeType(String name) throws IllegalNameException, NamespaceException {
		}

        @Override
        public void build() throws IllegalStateException {
		}
    }

    private class QNodeDefinitionBuilderImpl extends AbstractNodeDefinitionBuilder<QNodeTypeDefinition> {
        private final QNodeTypeDefinitionBuilderImpl ntd;

        private final QNodeDefinitionBuilder builder = new QNodeDefinitionBuilder();

        public QNodeDefinitionBuilderImpl(QNodeTypeDefinitionBuilderImpl ntd) {
            super();
            this.ntd = ntd;
        }

        @Override
        public void setName(String name) throws RepositoryException {
		}

        @Override
        public void setAllowsSameNameSiblings(boolean allowSns)
                throws RepositoryException {
				}

        @Override
        public void setAutoCreated(boolean autocreate)
                throws RepositoryException {
				}

        @Override
        public void setOnParentVersion(int onParent)
                throws RepositoryException {
				}

        @Override
        public void setProtected(boolean isProtected)
                throws RepositoryException {
				}

        @Override
        public void setMandatory(boolean isMandatory)
                throws RepositoryException {
				}

        @Override
        public void addRequiredPrimaryType(String name) throws IllegalNameException, NamespaceException {
		}

        @Override
        public void setDefaultPrimaryType(String name) throws IllegalNameException, NamespaceException {
		}

        @Override
        public void setDeclaringNodeType(String name) throws IllegalNameException, NamespaceException {
		}

        @Override
        public void build() {
		}
    }


    private Name toName(String name) throws IllegalNameException, NamespaceException {
		return null;
	}

}
