/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jackrabbit.vault.util.xml.xerces.util;

import java.util.Enumeration;
import java.util.NoSuchElementException;

import org.apache.jackrabbit.vault.util.xml.xerces.xni.NamespaceContext;

/**
 * Namespace support for XML document handlers. This class doesn't
 * perform any error checking and assumes that all strings passed
 * as arguments to methods are unique symbols. The SymbolTable class
 * can be used for this purpose.
 *
 * @author Andy Clark, IBM
 * @version $Id$
 */
public class NamespaceSupport implements NamespaceContext {

    //
    // Data
    //

    /**
     * Namespace binding information. This array is composed of a
     * series of tuples containing the namespace binding information:
     * &lt;prefix, uri&gt;. The default size can be set to anything
     * as long as it is a power of 2 greater than 1.
     *
     * @see #fNamespaceSize
     * @see #fContext
     */
    protected String[] fNamespace = new String[16 * 2];

    /**
     * The top of the namespace information array.
     */
    protected int fNamespaceSize;

    // NOTE: The constructor depends on the initial context size 
    //       being at least 1. -Ac

    /**
     * Context indexes. This array contains indexes into the namespace
     * information array. The index at the current context is the start
     * index of declared namespace bindings and runs to the size of the
     * namespace information array.
     *
     * @see #fNamespaceSize
     */
    protected int[] fContext = new int[8];

    /**
     * The current context.
     */
    protected int fCurrentContext;

    protected String[] fPrefixes = new String[16];

    //
    // Constructors
    //

    /**
     * Default constructor.
     */
    public NamespaceSupport() {
    } // <init>()

    /**
     * Constructs a namespace context object and initializes it with
     * the prefixes declared in the specified context.
     */
    public NamespaceSupport(NamespaceContext context) {
        pushContext();
        // copy declaration in the context
        Enumeration prefixes = context.getAllPrefixes();
        while (prefixes.hasMoreElements()) {
            String prefix = (String) prefixes.nextElement();
            String uri = context.getURI(prefix);
            declarePrefix(prefix, uri);
        }
    } // <init>(NamespaceContext)


    //
    // Public methods
    //

    /**
     * @see org.apache.jackrabbit.vault.util.xml.xerces.xni.NamespaceContext#reset()
     */
    public void reset() {
	} // reset(SymbolTable)


    /**
     * @see org.apache.jackrabbit.vault.util.xml.xerces.xni.NamespaceContext#pushContext()
     */
    public void pushContext() {
	} // pushContext()


    /**
     * @see org.apache.jackrabbit.vault.util.xml.xerces.xni.NamespaceContext#popContext()
     */
    public void popContext() {
	} // popContext()

    /**
     * @see org.apache.jackrabbit.vault.util.xml.xerces.xni.NamespaceContext#declarePrefix(String, String)
     */
    public boolean declarePrefix(String prefix, String uri) {
		return false;
	} // declarePrefix(String,String):boolean

    /**
     * @see org.apache.jackrabbit.vault.util.xml.xerces.xni.NamespaceContext#getURI(String)
     */
    public String getURI(String prefix) {
		return null;
	} // getURI(String):String


    /**
     * @see org.apache.jackrabbit.vault.util.xml.xerces.xni.NamespaceContext#getPrefix(String)
     */
    public String getPrefix(String uri) {
		return null;
	} // getPrefix(String):String


    /**
     * @see org.apache.jackrabbit.vault.util.xml.xerces.xni.NamespaceContext#getDeclaredPrefixCount()
     */
    public int getDeclaredPrefixCount() {
		return 0;
	} // getDeclaredPrefixCount():int

    /**
     * @see org.apache.jackrabbit.vault.util.xml.xerces.xni.NamespaceContext#getDeclaredPrefixAt(int)
     */
    public String getDeclaredPrefixAt(int index) {
		return null;
	} // getDeclaredPrefixAt(int):String

    /**
     * @see org.apache.jackrabbit.vault.util.xml.xerces.xni.NamespaceContext#getAllPrefixes()
     */
    public Enumeration getAllPrefixes() {
		return null;
	}

    protected final class Prefixes implements Enumeration {
        private String[] prefixes;
        private int counter = 0;
        private int size = 0;

        /**
         * Constructor for Prefixes.
         */
        public Prefixes(String[] prefixes, int size) {
            this.prefixes = prefixes;
            this.size = size;
        }

        /**
         * @see java.util.Enumeration#hasMoreElements()
         */
        public boolean hasMoreElements() {
			return false;
		}

        /**
         * @see java.util.Enumeration#nextElement()
         */
        public Object nextElement() {
			return null;
		}

        public String toString() {
			return null;
		}

}

} // class NamespaceSupport
