/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.bgservlets;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.security.Principal;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletInputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.sling.api.SlingException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/** Minimal HttpServletRequest for background processing */
public class BackgroundHttpServletRequest implements HttpServletRequest {

    private static final Logger log = LoggerFactory.getLogger(BackgroundHttpServletRequest.class);
    
    private final String contextPath;
    private final String method;
    private final String pathInfo;
    private final String servletPath;
    private final String queryString;
    private final String requestURI;
    private final StringBuffer requestURL;
    private String characterEncoding;
    private final int contentLength;
    private final String contentType;
    private final Locale locale;
    private final String protocol;
    private final String remoteAddr;
    private final String remoteHost;
    private final int remotePort;
    private final int serverPort;
    private final String scheme;
    private final String remoteUser;
    private final String serverName;

    private final Map<String, Object> attributes;
    private final Map<String, ?> parameters;

    static class IteratorEnumeration<T> implements Enumeration<T> {
        private final Iterator<T> it;

        IteratorEnumeration(Iterator<T> it) {
            this.it = it;
        }

        public boolean hasMoreElements() {
			return false;
		}

        public T nextElement() {
			return null;
		}
    }

    /**
     * We throw this for any method for which we do not have data that's safe to
     * use outside of the container's request/response cycle. Start by throwing
     * this everywhere and implement methods as needed, if their data is safe to
     * use.
     */
    @SuppressWarnings("serial")
    class UnsupportedBackgroundOperationException extends
            UnsupportedOperationException {
        UnsupportedBackgroundOperationException() {
            super("This operation is not supported for background requests");
        }
    }

    @SuppressWarnings("unchecked")
    public BackgroundHttpServletRequest(HttpServletRequest r,
            String[] parametersToRemove) {

        // Store objects which are safe to use outside
        // of the container's request/response cycle - the
        // goal is to release r once this request starts
        // executing in the background
        contextPath = r.getContextPath();
        method = r.getMethod();
        pathInfo = r.getPathInfo();
        servletPath = r.getServletPath();
        queryString = filterQueryString(r.getQueryString(), parametersToRemove);
        requestURI = r.getRequestURI();
        requestURL = r.getRequestURL();
        characterEncoding = r.getCharacterEncoding();
        contentLength = r.getContentLength();
        contentType = r.getContentType();
        locale = r.getLocale();
        protocol = r.getProtocol();
        remoteAddr = r.getRemoteAddr();
        remoteHost = r.getRemoteHost();
        remotePort = r.getRemotePort();
        serverPort = r.getServerPort();
        scheme = r.getScheme();
        remoteUser = r.getRemoteUser();
        serverName = r.getServerName();

        attributes = new HashMap<String, Object>();
        //
        // Don't copy attributes, we consider this to be a "fresh" request final
        // Enumeration<?> e = r.getAttributeNames(); while(e.hasMoreElements())
        // { final String key = (String)e.nextElement(); attributes.put(key,
        // r.getAttribute(key)); }
        //

        parameters = new HashMap<String, String>();
        parameters.putAll(r.getParameterMap());
        for (String key : parametersToRemove) {
            parameters.remove(key);
        }
    }
    
    private String filterQueryString(String original, String [] parametersToRemove) {
		return null;
	}

    public String getAuthType() {
		return null;
	}

    public String getContextPath() {
		return null;
	}

    public Cookie[] getCookies() {
		return null;
	}

    public long getDateHeader(String arg0) {
		return 0L;
	}

    public String getHeader(String arg0) {
		return null;
	}

    public Enumeration<?> getHeaderNames() {
		return null;
	}

    public Enumeration<?> getHeaders(String name) {
		return null;
	}

    public int getIntHeader(String name) {
		return 0;
	}

    public String getMethod() {
		return null;
	}

    public String getPathInfo() {
		return null;
	}

    public String getPathTranslated() {
		return null;
	}

    public String getQueryString() {
		return null;
	}

    public String getRemoteUser() {
		return null;
	}

    public String getRequestedSessionId() {
		return null;
	}

    public String getRequestURI() {
		return null;
	}

    public StringBuffer getRequestURL() {
		return null;
	}

    public String getServletPath() {
		return null;
	}

    public HttpSession getSession() {
		return null;
	}

    public HttpSession getSession(boolean arg0) {
		return null;
	}

    public Principal getUserPrincipal() {
		return null;
	}

    public boolean isRequestedSessionIdFromCookie() {
		return false;
	}

    public boolean isRequestedSessionIdFromUrl() {
		return false;
	}

    public boolean isRequestedSessionIdFromURL() {
		return false;
	}

    public boolean isRequestedSessionIdValid() {
		return false;
	}

    public boolean isUserInRole(String arg0) {
		return false;
	}

    public Object getAttribute(String name) {
		return null;
	}

    public Enumeration<?> getAttributeNames() {
		return null;
	}

    public String getCharacterEncoding() {
		return null;
	}

    public int getContentLength() {
		return 0;
	}

    public String getContentType() {
		return null;
	}

    private static final String encode(final String value) {
		return null;
	}

    private boolean append(final StringBuilder sb, final String key, final String val, final boolean first) {
		return false;
	}

    public ServletInputStream getInputStream() throws IOException {
		return null;
	}

    private static final class ByteArrayServletInputStream extends ServletInputStream {

        private final ByteArrayInputStream stream;

        public ByteArrayServletInputStream(final ByteArrayInputStream stream) {
            this.stream = stream;
        }

        @Override
        public int available() throws IOException {
			return 0;
		}

        @Override
        public void close() throws IOException {
		}

        @Override
        public synchronized void mark(final int arg0) {
		}

        @Override
        public boolean markSupported() {
			return false;
		}

        @Override
        public int read() throws IOException {
			return 0;
		}

        @Override
        public int read(final byte[] arg0, final int arg1, final int arg2) throws IOException {
			return 0;
		}

        @Override
        public int read(final byte[] arg0) throws IOException {
			return 0;
		}

        @Override
        public synchronized void reset() throws IOException {
		}

        @Override
        public long skip(final long arg0) throws IOException {
			return 0L;
		}
    }

    public String getLocalAddr() {
		return null;
	}

    public Locale getLocale() {
		return null;
	}

    public Enumeration<?> getLocales() {
		return null;
	}

    public String getLocalName() {
		return null;
	}

    public int getLocalPort() {
		return 0;
	}

    public String getParameter(String name) {
		return null;
	}

    public Map<?, ?> getParameterMap() {
		return null;
	}

    public Enumeration<?> getParameterNames() {
		return null;
	}

    public String[] getParameterValues(String key) {
		return null;
	}

    public String getProtocol() {
		return null;
	}

    public BufferedReader getReader() throws IOException {
		return null;
	}

    public String getRealPath(String arg0) {
		return null;
	}

    public String getRemoteAddr() {
		return null;
	}

    public String getRemoteHost() {
		return null;
	}

    public int getRemotePort() {
		return 0;
	}

    public RequestDispatcher getRequestDispatcher(String arg0) {
		return null;
	}

    public String getScheme() {
		return null;
	}

    public String getServerName() {
		return null;
	}

    public int getServerPort() {
		return 0;
	}

    public boolean isSecure() {
		return false;
	}

    public void removeAttribute(String name) {
	}

    public void setAttribute(String key, Object value) {
	}

    public void setCharacterEncoding(final String encoding)
            throws UnsupportedEncodingException {
			}
}
