/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.sling.scripting.jsp.jasper.runtime;

import java.io.IOException;
import java.io.Writer;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.util.Enumeration;
import java.util.HashMap;

import javax.el.ELContext;
import javax.el.ExpressionFactory;
import javax.el.ValueExpression;
import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspFactory;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.el.ELException;
import javax.servlet.jsp.el.ExpressionEvaluator;
import javax.servlet.jsp.el.VariableResolver;
import javax.servlet.jsp.tagext.BodyContent;

import org.apache.juli.logging.Log;
import org.apache.juli.logging.LogFactory;
import org.apache.sling.scripting.jsp.SlingPageException;
import org.apache.sling.scripting.jsp.jasper.Constants;
import org.apache.sling.scripting.jsp.jasper.compiler.Localizer;
import org.apache.sling.scripting.jsp.jasper.el.ELContextImpl;
import org.apache.sling.scripting.jsp.jasper.el.ExpressionEvaluatorImpl;
import org.apache.sling.scripting.jsp.jasper.el.FunctionMapperImpl;
import org.apache.sling.scripting.jsp.jasper.el.VariableResolverImpl;
import org.apache.sling.scripting.jsp.jasper.security.SecurityUtil;
import org.apache.sling.scripting.jsp.jasper.util.Enumerator;

/**
 * Implementation of the PageContext class from the JSP spec. Also doubles as a
 * VariableResolver for the EL.
 *
 * @author Anil K. Vijendran
 * @author Larry Cable
 * @author Hans Bergsten
 * @author Pierre Delisle
 * @author Mark Roth
 * @author Jan Luehe
 * @author Jacob Hookom
 */
public class PageContextImpl extends PageContext {

	private Log log = LogFactory.getLog(PageContextImpl.class);

	private BodyContentImpl[] outs;

	private int depth;

	// per-servlet state
	private Servlet servlet;

	private ServletConfig config;

	private ServletContext context;

	private JspApplicationContextImpl applicationContext;

	private String errorPageURL;

	// page-scope attributes
	private transient HashMap<String, Object> attributes;

	// per-request state
	private transient ServletRequest request;

	private transient ServletResponse response;

	private transient HttpSession session;

	private transient ELContextImpl elContext;

	private boolean isIncluded;


	// initial output stream
	private transient JspWriter out;

	private transient JspWriterImpl baseOut;

	/*
	 * Constructor.
	 */
	PageContextImpl() {
		this.outs = new BodyContentImpl[0];
		this.attributes = new HashMap<String, Object>(16);
		this.depth = -1;
	}

	public void initialize(Servlet servlet, ServletRequest request,
			ServletResponse response, String errorPageURL,
			boolean needsSession, int bufferSize, boolean autoFlush)
			throws IOException {
			}

	private void _initialize(Servlet servlet, ServletRequest request,
			ServletResponse response, String errorPageURL,
			boolean needsSession, int bufferSize, boolean autoFlush)
			throws IOException {
			}

	public void release() {
	}

	public Object getAttribute(final String name) {
		return null;
	}

	private Object doGetAttribute(String name) {
		return null;
	}

	public Object getAttribute(final String name, final int scope) {
		return null;
	}

	private Object doGetAttribute(String name, int scope) {
		return null;
	}

	public void setAttribute(final String name, final Object attribute) {
	}

	private void doSetAttribute(String name, Object attribute) {
	}

	public void setAttribute(final String name, final Object o, final int scope) {
	}

	private void doSetAttribute(String name, Object o, int scope) {
	}

	public void removeAttribute(final String name, final int scope) {
	}

	private void doRemoveAttribute(String name, int scope) {
	}

	public int getAttributesScope(final String name) {
		return 0;
	}

	private int doGetAttributeScope(String name) {
		return 0;
	}

	public Object findAttribute(final String name) {
		return null;
	}

	private Object doFindAttribute(String name) {
		return null;
	}

	public Enumeration<String> getAttributeNamesInScope(final int scope) {
		return null;
	}

	private Enumeration doGetAttributeNamesInScope(int scope) {
		return null;
	}

	public void removeAttribute(final String name) {
	}

	private void doRemoveAttribute(String name) {
	}

	public JspWriter getOut() {
		return null;
	}

	public HttpSession getSession() {
		return null;
	}

	public Servlet getServlet() {
		return null;
	}

	public ServletConfig getServletConfig() {
		return null;
	}

	public ServletContext getServletContext() {
		return null;
	}

	public ServletRequest getRequest() {
		return null;
	}

	public ServletResponse getResponse() {
		return null;
	}

	/**
	 * Returns the exception associated with this page context, if any. <p/>
	 * Added wrapping for Throwables to avoid ClassCastException: see Bugzilla
	 * 31171 for details.
	 *
	 * @return The Exception associated with this page context, if any.
	 */
	public Exception getException() {
		return null;
	}

	public Object getPage() {
		return null;
	}

	private final String getAbsolutePathRelativeToContext(String relativeUrlPath) {
		return null;
	}

	public void include(String relativeUrlPath) throws ServletException,
			IOException {
			}

	public void include(final String relativeUrlPath, final boolean flush)
			throws ServletException, IOException {
			}

	private void doInclude(String relativeUrlPath, boolean flush)
			throws ServletException, IOException {
			}

	public VariableResolver getVariableResolver() {
		return null;
	}

	public void forward(final String relativeUrlPath) throws ServletException,
			IOException {
			}

	private void doForward(String relativeUrlPath) throws ServletException,
			IOException {
			}

    public void forwardToErrorPage(final String relativeUrlPath) throws ServletException,
            IOException {
			}

    private void doForwardToErrorPage(String relativeUrlPath) throws ServletException,
            IOException {
			}

	public BodyContent pushBody() {
		return null;
	}

	public JspWriter pushBody(Writer writer) {
		return null;
	}

	public JspWriter popBody() {
		return null;
	}

	/**
	 * Provides programmatic access to the ExpressionEvaluator. The JSP
	 * Container must return a valid instance of an ExpressionEvaluator that can
	 * parse EL expressions.
	 */
	public ExpressionEvaluator getExpressionEvaluator() {
		return null;
	}

	public void handlePageException(Exception ex) throws IOException,
			ServletException {
			}

	public void handlePageException(final Throwable t) throws IOException,
			ServletException {
			}

	private void doHandlePageException(Throwable t) throws IOException,
			ServletException {
			}

	private static String XmlEscape(String s) {
		return null;
	}

	/**
	 * Proprietary method to evaluate EL expressions. XXX - This method should
	 * go away once the EL interpreter moves out of JSTL and into its own
	 * project. For now, this is necessary because the standard machinery is too
	 * slow.
	 *
	 * @param expression
	 *            The expression to be evaluated
	 * @param expectedType
	 *            The expected resulting type
	 * @param pageContext
	 *            The page context
	 * @param functionMap
	 *            Maps prefix and name to Method
	 * @return The result of the evaluation
	 */
	public static Object proprietaryEvaluate(final String expression,
			final Class expectedType, final PageContext pageContext,
			final ProtectedFunctionMapper functionMap, final boolean escape)
			throws ELException {
				return null;
			}

	public ELContext getELContext() {
		return null;
	}

}
